@@ -9,7 +9,8 @@ osmo-build-dep.sh libosmocore "" --disable-doxygen
 export PKG_CONFIG_PATH="$inst/lib/pkgconfig:$PKG_CONFIG_PATH"
 export LD_LIBRARY_PATH="$inst/lib"
 
-osmo-build-dep.sh libosmo-abis
+osmo-build-dep.sh libosmo-abis "" --disable-dahdi
+osmo-build-dep.sh libosmo-netif "" --disable-doxygen
 
 cd "$deps"
 osmo-layer1-headers.sh lc15 "$FIRMWARE_VERSION"
