@@ -12,11 +12,9 @@ enum bts_vty_node {
 	PHY_INST_NODE,
 	BTS_NODE,
 	TRX_NODE,
+	OSMUX_NODE,
 };
 
-extern struct cmd_element ournode_exit_cmd;
-extern struct cmd_element ournode_end_cmd;
-
 extern struct cmd_element cfg_bts_auto_band_cmd;
 extern struct cmd_element cfg_bts_no_auto_band_cmd;
 
@@ -25,10 +23,15 @@ struct phy_instance *vty_get_phy_instance(struct vty *vty, int phy_nr, int inst_
 int bts_vty_go_parent(struct vty *vty);
 int bts_vty_is_config_node(struct vty *vty, int node);
 
-int bts_vty_init(struct gsm_bts *bts);
-
-struct gsm_network *gsmnet_from_vty(struct vty *v);
+int bts_vty_init(void *ctx);
 
 extern struct vty_app_info bts_vty_info;
+extern struct gsm_bts *g_bts;
+
+enum bts_vty_cmd_attr {
+	BTS_VTY_ATTR_NEW_LCHAN,
+	BTS_VTY_TRX_POWERCYCLE,
+	/* NOTE: up to 32 entries */
+};
 
 #endif
