@@ -72,9 +72,16 @@ bool ts45008_83_is_sub(struct gsm_lchan *lchan, uint32_t fn)
 			/* No DTX allowed; SUB=FULL, therefore measurements at all frame numbers are
 			 * SUB */
 			return true;
+		case GSM48_CMODE_DATA_12k0: /* TCH/F9.6 */
+		case GSM48_CMODE_DATA_6k0: /* TCH/F4.8 */
+			/* FIXME: In case of data traffic channels TCH/F9.6 and TCH/F4.8 the
+			 * RXQUAL_SUB report shall include measurements on the TDMA frames given
+			 * in the table of subclause 8.3 only if L2 fill frames have been received
+			 * as FACCH/F frames at the corresponding frame positions. */
 		default:
-			LOGPFN(DMEAS, LOGL_ERROR, fn, "%s: Unsupported lchan->tch_mode %u\n",
-				gsm_lchan_name(lchan), lchan->tch_mode);
+			if (lchan->rsl_cmode == RSL_CMOD_SPD_DATA)
+				return false;
+			LOGPLCFN(lchan, fn, DMEAS, LOGL_ERROR, "Unsupported lchan->tch_mode %u\n", lchan->tch_mode);
 			break;
 		}
 		break;
@@ -88,9 +95,16 @@ bool ts45008_83_is_sub(struct gsm_lchan *lchan, uint32_t fn)
 			/* No DTX allowed; SUB=FULL, therefore measurements at all frame numbers are
 			 * SUB */
 			return true;
+		case GSM48_CMODE_DATA_6k0: /* TCH/H4.8 */
+		case GSM48_CMODE_DATA_3k6: /* TCH/H2.4 */
+			/* FIXME: In case of data traffic channels TCH/H4.8 and TCH/H2.4 the
+			 * RXQUAL_SUB report shall include measurements on the TDMA frames given
+			 * in the table of subclause 8.3 only if L2 fill frames have been received
+			 * as FACCH/H frames at the corresponding frame positions. */
 		default:
-			LOGPFN(DMEAS, LOGL_ERROR, fn, "%s: Unsupported lchan->tch_mode %u\n",
-				gsm_lchan_name(lchan), lchan->tch_mode);
+			if (lchan->rsl_cmode == RSL_CMOD_SPD_DATA)
+				return false;
+			LOGPLCFN(lchan, fn, DMEAS, LOGL_ERROR, "Unsupported lchan->tch_mode %u\n", lchan->tch_mode);
 			break;
 		}
 		break;
@@ -269,9 +283,8 @@ int is_meas_complete(struct gsm_lchan *lchan, uint32_t fn)
 	}
 
 	if (rc == 1) {
-		DEBUGP(DMEAS,
-		       "%s meas period end fn:%u, fn_mod:%i, status:%d, pchan:%s\n",
-		       gsm_lchan_name(lchan), fn, fn_mod, rc, gsm_pchan_name(pchan));
+		LOGPLCFN(lchan, fn, DMEAS, LOGL_DEBUG, "meas period end fn_mod:%d, status:%d, pchan:%s\n", fn_mod,
+			 rc, gsm_pchan_name(pchan));
 	}
 
 	return rc;
@@ -307,16 +320,15 @@ int lchan_new_ul_meas(struct gsm_lchan *lchan,
 	struct bts_ul_meas *dest;
 
 	if (lchan->state != LCHAN_S_ACTIVE) {
-		LOGPFN(DMEAS, LOGL_NOTICE, fn,
-		       "%s measurement during state: %s, num_ul_meas=%d, fn_mod=%u\n",
-		       gsm_lchan_name(lchan), gsm_lchans_name(lchan->state),
-		       lchan->meas.num_ul_meas, fn_mod);
+		LOGPLCFN(lchan, fn, DMEAS, LOGL_NOTICE,
+			 "measurement during state: %s, num_ul_meas=%d, fn_mod=%u\n",
+			 gsm_lchans_name(lchan->state), lchan->meas.num_ul_meas, fn_mod);
 	}
 
 	if (lchan->meas.num_ul_meas >= ARRAY_SIZE(lchan->meas.uplink)) {
-		LOGPFN(DMEAS, LOGL_NOTICE, fn,
-		       "%s no space for uplink measurement, num_ul_meas=%d, fn_mod=%u\n",
-		       gsm_lchan_name(lchan), lchan->meas.num_ul_meas, fn_mod);
+		LOGPLCFN(lchan, fn, DMEAS, LOGL_NOTICE,
+			 "no space for uplink measurement, num_ul_meas=%d, fn_mod=%u\n", lchan->meas.num_ul_meas,
+			 fn_mod);
 		return -ENOSPC;
 	}
 
@@ -328,10 +340,9 @@ int lchan_new_ul_meas(struct gsm_lchan *lchan,
 	if (!ulm->is_sub)
 		dest->is_sub = ts45008_83_is_sub(lchan, fn);
 
-	DEBUGPFN(DMEAS, fn, "%s adding a %s measurement (ber10k=%u, ta_offs=%d, ci_cB=%d, rssi=-%u), num_ul_meas=%d, fn_mod=%u\n",
-		 gsm_lchan_name(lchan),
-		 dest->is_sub ? "SUB" : "FULL",
-		 ulm->ber10k, ulm->ta_offs_256bits, ulm->ci_cb, ulm->inv_rssi,
+	LOGPLCFN(lchan, fn, DMEAS, LOGL_DEBUG,
+		 "adding a %s measurement (ber10k=%u, ta_offs=%d, ci_cB=%d, rssi=-%u), num_ul_meas=%d, fn_mod=%u\n",
+		 dest->is_sub ? "SUB" : "FULL", ulm->ber10k, ulm->ta_offs_256bits, ulm->ci_cb, ulm->inv_rssi,
 		 lchan->meas.num_ul_meas, fn_mod);
 
 	lchan->meas.last_fn = fn;
