@@ -1,6 +1,6 @@
 /* NM Radio Carrier FSM */
 
-/* (C) 2020 by sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+/* (C) 2020 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Pau Espin Pedrol <pespin@sysmocom.de>
  *
  * All Rights Reserved
@@ -70,17 +70,17 @@ static void st_op_disabled_notinstalled_on_enter(struct osmo_fsm_inst *fi, uint3
 static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct gsm_bts_bb_trx *bb_transc = (struct gsm_bts_bb_trx *)fi->priv;
-	struct gsm_bts_trx *trx = gsm_bts_bb_trx_get_trx(bb_transc);
-	int i;
 
 	switch (event) {
 	case NM_EV_SW_ACT:
 		oml_mo_tx_sw_act_rep(&bb_transc->mo);
 		nm_bb_transc_fsm_state_chg(fi, NM_BBTRANSC_ST_OP_DISABLED_OFFLINE);
-		for (i = 0; i < TRX_NR_TS; i++) {
-			struct gsm_bts_trx_ts *ts = &trx->ts[i];
-			osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_BBTRANSC_INSTALLED, NULL);
-		}
+		ev_dispatch_children(bb_transc, event);
+		return;
+	case NM_EV_OML_UP:
+		/* Report current state: */
+		oml_tx_state_changed(&bb_transc->mo);
+		ev_dispatch_children(bb_transc, event);
 		return;
 	case NM_EV_RSL_UP:
 		return;
@@ -122,14 +122,31 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 	struct nm_fsm_ev_setattr_data *setattr_data;
 	bool phy_state_connected;
 	bool rsl_link_connected;
+	int rc;
 
 	switch (event) {
-	case NM_EV_SETATTR_ACK:
-	case NM_EV_SETATTR_NACK:
+	case NM_EV_OML_UP:
+		/* Report current state: */
+		oml_tx_state_changed(&bb_transc->mo);
+		ev_dispatch_children(bb_transc, event);
+		return;
+	case NM_EV_RX_SETATTR:
 		setattr_data = (struct nm_fsm_ev_setattr_data *)data;
-		bb_transc->mo.setattr_success = setattr_data->cause == 0;
-		oml_fom_ack_nack(setattr_data->msg, setattr_data->cause);
-		break;
+		rc = bts_model_apply_oml(trx->bts, setattr_data->msg,
+					 &bb_transc->mo, bb_transc);
+		bb_transc->mo.setattr_success = rc == 0;
+		oml_fom_ack_nack_copy_msg(setattr_data->msg, rc);
+		return;
+	case NM_EV_RX_OPSTART:
+#if 0
+		/* Disabled because osmo-bsc doesn't send SetAttr on BB_TRANSC object */
+		if (!bb_transc->mo.setattr_success) {
+			oml_mo_opstart_nack(&bb_transc->mo, NM_NACK_CANT_PERFORM);
+			return;
+		}
+#endif
+		bts_model_opstart(trx->bts, &bb_transc->mo, bb_transc);
+		return;
 	case NM_EV_OPSTART_ACK:
 		bb_transc->mo.opstart_success = true;
 		oml_mo_opstart_ack(&bb_transc->mo);
@@ -231,6 +248,7 @@ static struct osmo_fsm_state nm_bb_transc_fsm_states[] = {
 	[NM_BBTRANSC_ST_OP_DISABLED_NOTINSTALLED] = {
 		.in_event_mask =
 			X(NM_EV_SW_ACT) |
+			X(NM_EV_OML_UP) |
 			X(NM_EV_RSL_UP) |
 			X(NM_EV_RSL_DOWN) |
 			X(NM_EV_PHYLINK_UP) |
@@ -245,8 +263,9 @@ static struct osmo_fsm_state nm_bb_transc_fsm_states[] = {
 	},
 	[NM_BBTRANSC_ST_OP_DISABLED_OFFLINE] = {
 		.in_event_mask =
-			X(NM_EV_SETATTR_ACK) |
-			X(NM_EV_SETATTR_NACK) |
+			X(NM_EV_OML_UP) |
+			X(NM_EV_RX_SETATTR) |
+			X(NM_EV_RX_OPSTART) |
 			X(NM_EV_OPSTART_ACK) |
 			X(NM_EV_OPSTART_NACK) |
 			X(NM_EV_RSL_UP) |
