@@ -1,6 +1,6 @@
 /* NM BTS FSM */
 
-/* (C) 2020 by sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+/* (C) 2020 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Pau Espin Pedrol <pespin@sysmocom.de>
  *
  * All Rights Reserved
@@ -36,6 +36,7 @@
 #include <osmo-bts/nm_common_fsm.h>
 #include <osmo-bts/phy_link.h>
 #include <osmo-bts/cbch.h>
+#include <osmo-bts/notification.h>
 
 #define X(s) (1 << (s))
 
@@ -60,8 +61,13 @@ static void st_op_disabled_notinstalled_on_enter(struct osmo_fsm_inst *fi, uint3
 	struct gsm_bts *bts = (struct gsm_bts *)fi->priv;
 	/* Reset state: */
 	bts->si_valid = 0;
+	bts->bsic_configured = false;
+	bts->bsic = 0xff; /* invalid value */
 	TALLOC_FREE(bts->mo.nm_attr);
 	bts_cbch_reset(bts);
+	bts_asci_notification_reset(bts);
+	if (bts->c0_power_red_db > 0)
+		bts_set_c0_pwr_red(bts, 0);
 
 	bts->mo.setattr_success = false;
 	bts->mo.opstart_success = false;
@@ -74,7 +80,8 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 	struct gsm_bts_trx *trx;
 
 	switch (event) {
-	case NM_EV_SW_ACT:
+	case NM_EV_OML_UP:
+		/* automatic SW_ACT upon OML link establishment: */
 		oml_mo_tx_sw_act_rep(&bts->mo);
 
 		llist_for_each_entry(trx, &bts->trx_list, list) {
@@ -91,6 +98,7 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 		}
 
 		nm_bts_fsm_state_chg(fi, NM_BTS_ST_OP_DISABLED_OFFLINE);
+		ev_dispatch_children(bts, event);
 		return;
 	default:
 		OSMO_ASSERT(0);
@@ -109,13 +117,21 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 {
 	struct gsm_bts *bts = (struct gsm_bts *)fi->priv;
 	struct nm_fsm_ev_setattr_data *setattr_data;
+	int rc;
 
 	switch (event) {
-	case NM_EV_SETATTR_ACK:
-	case NM_EV_SETATTR_NACK:
+	case NM_EV_RX_SETATTR:
 		setattr_data = (struct nm_fsm_ev_setattr_data *)data;
-		bts->mo.setattr_success = setattr_data->cause == 0;
-		oml_fom_ack_nack(setattr_data->msg, setattr_data->cause);
+		rc = bts_model_apply_oml(bts, setattr_data->msg, &bts->mo, bts);
+		bts->mo.setattr_success = rc == 0;
+		oml_fom_ack_nack_copy_msg(setattr_data->msg, rc);
+		break;
+	case NM_EV_RX_OPSTART:
+		if (!bts->mo.setattr_success) {
+			oml_mo_opstart_nack(&bts->mo, NM_NACK_CANT_PERFORM);
+			return;
+		}
+		bts_model_opstart(bts, &bts->mo, bts);
 		break;
 	case NM_EV_OPSTART_ACK:
 		bts->mo.opstart_success = true;
@@ -166,7 +182,7 @@ static void nm_bts_allstate(struct osmo_fsm_inst *fi, uint32_t event, void *data
 static struct osmo_fsm_state nm_bts_fsm_states[] = {
 	[NM_BTS_ST_OP_DISABLED_NOTINSTALLED] = {
 		.in_event_mask =
-			X(NM_EV_SW_ACT),
+			X(NM_EV_OML_UP),
 		.out_state_mask =
 			X(NM_BTS_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_BTS_ST_OP_DISABLED_OFFLINE),
@@ -176,8 +192,8 @@ static struct osmo_fsm_state nm_bts_fsm_states[] = {
 	},
 	[NM_BTS_ST_OP_DISABLED_OFFLINE] = {
 		.in_event_mask =
-			X(NM_EV_SETATTR_ACK) |
-			X(NM_EV_SETATTR_NACK) |
+			X(NM_EV_RX_SETATTR) |
+			X(NM_EV_RX_OPSTART) |
 			X(NM_EV_OPSTART_ACK) |
 			X(NM_EV_OPSTART_NACK),
 		.out_state_mask =
