@@ -1,6 +1,6 @@
 /* NM Radio Carrier FSM */
 
-/* (C) 2020 by sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+/* (C) 2020 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Pau Espin Pedrol <pespin@sysmocom.de>
  *
  * All Rights Reserved
@@ -48,8 +48,12 @@
 static void st_op_disabled_notinstalled_on_enter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
+	/* Reset state: */
+	TALLOC_FREE(trx->mo.nm_attr);
+
+	trx->mo.setattr_success = false;
 	trx->mo.opstart_success = false;
-	oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_NOT_INSTALLED);
+	oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_NOT_INSTALLED, NM_STATE_LOCKED);
 }
 
 static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
@@ -61,6 +65,10 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 		oml_mo_tx_sw_act_rep(&trx->mo);
 		nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_OFFLINE);
 		return;
+	case NM_EV_OML_UP:
+		/* Report current state: */
+		oml_tx_state_changed(&trx->mo);
+		return;
 	case NM_EV_RSL_UP:
 		return;
 	case NM_EV_RSL_DOWN:
@@ -79,17 +87,47 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 static void st_op_disabled_offline_on_enter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
+	unsigned int i;
+
+	trx->mo.setattr_success = false;
 	trx->mo.opstart_success = false;
-	oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE);
+	oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE, -1);
+
+	if (prev_state == NM_RCARRIER_ST_OP_ENABLED) {
+		for (i = 0; i < TRX_NR_TS; i++) {
+			struct gsm_bts_trx_ts *ts = &trx->ts[i];
+			osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_RCARRIER_DISABLED, NULL);
+		}
+	}
 }
 
 static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
+	struct nm_fsm_ev_setattr_data *setattr_data;
 	bool phy_state_connected;
 	bool rsl_link_connected;
+	int rc;
 
 	switch (event) {
+	case NM_EV_OML_UP:
+		/* Report current state: */
+		oml_tx_state_changed(&trx->mo);
+		return;
+	case NM_EV_RX_SETATTR:
+		setattr_data = (struct nm_fsm_ev_setattr_data *)data;
+		rc = bts_model_apply_oml(trx->bts, setattr_data->msg,
+					 &trx->mo, trx);
+		trx->mo.setattr_success = rc == 0;
+		oml_fom_ack_nack_copy_msg(setattr_data->msg, rc);
+		break; /* check statechg below */
+	case NM_EV_RX_OPSTART:
+		if (!trx->mo.setattr_success) {
+			oml_mo_opstart_nack(&trx->mo, NM_NACK_CANT_PERFORM);
+			return;
+		}
+		bts_model_opstart(trx->bts, &trx->mo, trx);
+		return;
 	case NM_EV_OPSTART_ACK:
 		trx->mo.opstart_success = true;
 		oml_mo_opstart_ack(&trx->mo);
@@ -122,13 +160,14 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 	}
 
 	if (rsl_link_connected && phy_state_connected &&
-	    trx->mo.opstart_success) {
+	    trx->mo.setattr_success && trx->mo.opstart_success) {
 		nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_ENABLED);
 	} else {
-		LOGPFSML(fi, LOGL_INFO, "Delay switch to operative state Enabled, wait for:%s%s%s\n",
-			 rsl_link_connected ? "": " rsl",
-			 phy_state_connected ? "": " phy",
-			 trx->mo.opstart_success ? "": " opstart");
+		LOGPFSML(fi, LOGL_INFO, "Delay switch to operative state Enabled, wait for:%s%s%s%s\n",
+			 rsl_link_connected ? "" : " rsl",
+			 phy_state_connected ? "" : " phy",
+			 trx->mo.setattr_success ? "" : " setattr",
+			 trx->mo.opstart_success ? "" : " opstart");
 
 	}
 }
@@ -136,7 +175,14 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 static void st_op_enabled_on_enter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
-	oml_mo_state_chg(&trx->mo, NM_OPSTATE_ENABLED, NM_AVSTATE_OK);
+	unsigned int tn;
+
+	oml_mo_state_chg(&trx->mo, NM_OPSTATE_ENABLED, NM_AVSTATE_OK, -1);
+	/* Mark Dependency TS as Offline (ready to be Opstarted) */
+	for (tn = 0; tn < TRX_NR_TS; tn++) {
+		struct gsm_bts_trx_ts *ts = &trx->ts[tn];
+		osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_RCARRIER_ENABLED, NULL);
+	}
 }
 
 static void st_op_enabled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
@@ -155,15 +201,35 @@ static void st_op_enabled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 	nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_OFFLINE);
 }
 
+static void nm_rcarrier_allstate(struct osmo_fsm_inst *fi, uint32_t event, void *data)
+{
+	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
+
+	switch (event) {
+	case NM_EV_SHUTDOWN_START:
+		/* Announce we start shutting down */
+		oml_mo_state_chg(&trx->mo, -1, -1, NM_STATE_SHUTDOWN);
+		break;
+	case NM_EV_SHUTDOWN_FINISH:
+		nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED);
+		break;
+	default:
+		OSMO_ASSERT(false);
+	}
+}
+
 static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 	[NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED] = {
 		.in_event_mask =
 			X(NM_EV_SW_ACT) |
+			X(NM_EV_OML_UP) |
 			X(NM_EV_RSL_UP) |
 			X(NM_EV_RSL_DOWN) |
 			X(NM_EV_PHYLINK_UP) |
-			X(NM_EV_PHYLINK_DOWN),
+			X(NM_EV_PHYLINK_DOWN) |
+			X(NM_EV_DISABLE),
 		.out_state_mask =
+			X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_RCARRIER_ST_OP_DISABLED_OFFLINE),
 		.name = "DISABLED_NOTINSTALLED",
 		.onenter = st_op_disabled_notinstalled_on_enter,
@@ -171,13 +237,18 @@ static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 	},
 	[NM_RCARRIER_ST_OP_DISABLED_OFFLINE] = {
 		.in_event_mask =
+			X(NM_EV_OML_UP) |
+			X(NM_EV_RX_SETATTR) |
+			X(NM_EV_RX_OPSTART) |
 			X(NM_EV_OPSTART_ACK) |
 			X(NM_EV_OPSTART_NACK) |
 			X(NM_EV_RSL_UP) |
 			X(NM_EV_RSL_DOWN) |
 			X(NM_EV_PHYLINK_UP) |
-			X(NM_EV_PHYLINK_DOWN),
+			X(NM_EV_PHYLINK_DOWN) |
+			X(NM_EV_DISABLE),
 		.out_state_mask =
+			X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_RCARRIER_ST_OP_ENABLED),
 		.name = "DISABLED_OFFLINE",
 		.onenter = st_op_disabled_offline_on_enter,
@@ -186,8 +257,10 @@ static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 	[NM_RCARRIER_ST_OP_ENABLED] = {
 		.in_event_mask =
 			X(NM_EV_RSL_DOWN) |
-			X(NM_EV_PHYLINK_DOWN),
+			X(NM_EV_PHYLINK_DOWN) |
+			X(NM_EV_DISABLE),
 		.out_state_mask =
+			X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_RCARRIER_ST_OP_DISABLED_OFFLINE),
 		.name = "ENABLED",
 		.onenter = st_op_enabled_on_enter,
@@ -200,6 +273,9 @@ struct osmo_fsm nm_rcarrier_fsm = {
 	.states = nm_rcarrier_fsm_states,
 	.num_states = ARRAY_SIZE(nm_rcarrier_fsm_states),
 	.event_names = nm_fsm_event_names,
+	.allstate_action = nm_rcarrier_allstate,
+	.allstate_event_mask = X(NM_EV_SHUTDOWN_START) |
+			       X(NM_EV_SHUTDOWN_FINISH),
 	.log_subsys = DOML,
 };
 
