@@ -164,9 +164,9 @@ int oml_mo_send_msg(const struct gsm_abis_mo *mo, struct msgb *msg, uint8_t msg_
 	return oml_send_msg(msg, 0);
 }
 
-/* Put NM_ATT_SW_CONFIG as per 9.4.61 "SW Configuration" */
-static int add_att_sw_config(struct msgb *msg, const struct gsm_abis_mo *mo)
+static inline void add_bts_attrs(struct msgb *msg, const struct gsm_bts *bts)
 {
+	uint16_t total_len = 0;
 	uint8_t *len;
 
 	/* Put NM_ATT_SW_CONFIG as per 9.4.61 "SW Configuration". */
@@ -175,39 +175,17 @@ static int add_att_sw_config(struct msgb *msg, const struct gsm_abis_mo *mo)
 	/* We don't know the length yet, so we update it later. */
 	len = msgb_put(msg, 2);
 
-	switch (mo->obj_class) {
-	case NM_OC_BTS:
-	{
-		const struct gsm_bts *bts = mo->bts;
+	total_len += abis_nm_put_sw_file(msg, "osmobts", PACKAGE_VERSION, true);
+	total_len += abis_nm_put_sw_file(msg, btsatttr2str(BTS_TYPE_VARIANT),
+					 btsvariant2str(bts->variant), true);
 
-		abis_nm_put_sw_file(msg, "osmobts", PACKAGE_VERSION, true);
-		abis_nm_put_sw_file(msg, btsatttr2str(BTS_TYPE_VARIANT),
-				    btsvariant2str(bts->variant), true);
-		if (strlen(bts->sub_model)) {
-			abis_nm_put_sw_file(msg, btsatttr2str(BTS_SUB_MODEL),
-					    bts->sub_model, true);
-		}
-		break;
-	}
-	case NM_OC_BASEB_TRANSC:
-	{
-		const struct gsm_bts_trx *trx = container_of(mo, struct gsm_bts_trx, bb_transc.mo);
-		const struct phy_instance *pinst = trx->pinst;
-		const char *phy_version;
-
-		phy_version = pinst && strlen(pinst->version) ? pinst->version : "Unknown";
-		abis_nm_put_sw_file(msg, btsatttr2str(TRX_PHY_VERSION), phy_version, true);
-		break;
-	}
-	default:
-		msgb_get(msg, 1 + 2); /* TL16 */
-		return -ENOTSUP;
+	if (strlen(bts->sub_model)) {
+		total_len += abis_nm_put_sw_file(msg, btsatttr2str(BTS_SUB_MODEL),
+						 bts->sub_model, true);
 	}
 
 	/* Finally, update the length */
-	osmo_store16be((uint16_t)(msg->tail - (len + 2)), len);
-
-	return 0;
+	osmo_store16be(total_len, len);
 }
 
 /* Add BTS features as 3GPP TS 52.021 §9.4.30 Manufacturer Id */
@@ -217,64 +195,97 @@ static inline void add_bts_feat(struct msgb *msg, const struct gsm_bts *bts)
 	msgb_tl16v_put(msg, NM_ATT_MANUF_ID, len, bts->features->data);
 }
 
-/* Add ip.access feature flags for the given MO */
-static int add_att_ipacc_features(struct msgb *msg, const struct gsm_abis_mo *mo)
+static inline void add_trx_attr(struct msgb *msg, const struct gsm_bts_trx *trx)
 {
-	const struct gsm_bts *bts = mo->bts;
-	const struct gsm_bts_trx *trx;
-	uint32_t val;
+	const struct phy_instance *pinst = trx_phy_instance(trx);
+	const char *phy_version;
+	uint16_t total_len;
 	uint8_t *len;
 
-	msgb_v_put(msg, NM_ATT_IPACC_SUPP_FEATURES);
+	/* Put NM_ATT_SW_CONFIG as per 9.4.61 "SW Configuration". */
+	msgb_v_put(msg, NM_ATT_SW_CONFIG);
 
 	/* We don't know the length yet, so we update it later. */
 	len = msgb_put(msg, 2);
 
-	switch (mo->obj_class) {
-	case NM_OC_BTS:
-		msgb_tv16_put(msg, NM_IPAC_EIE_MAX_TA, 1); /* TL16 */
-		msgb_put_u8(msg, (bts->support.max_ta >> 0) & 0xff);
-		break;
-	case NM_OC_RADIO_CARRIER:
-		trx = container_of(mo, struct gsm_bts_trx, mo);
-		msgb_tv16_put(msg, NM_IPAC_EIE_FREQ_BANDS, 1); /* TL16 */
-		msgb_put_u8(msg, (trx->support.freq_bands >> 0) & 0xff);
-		break;
-	case NM_OC_BASEB_TRANSC:
-		trx = container_of(mo, struct gsm_bts_trx, bb_transc.mo);
-		msgb_tv16_put(msg, NM_IPAC_EIE_CIPH_ALGOS, 1); /* TL16 */
-		msgb_put_u8(msg, bts->support.ciphers); /* LSB is A5/1 */
-
-		msgb_tv16_put(msg, NM_IPAC_EIE_CHAN_TYPES, 2); /* TL16 */
-		msgb_put_u8(msg, (trx->support.chan_types >> 0) & 0xff);
-		msgb_put_u8(msg, (trx->support.chan_types >> 8) & 0xff);
-
-		msgb_tv16_put(msg, NM_IPAC_EIE_CHAN_MODES, 3); /* TL16 */
-		msgb_put_u8(msg, (trx->support.chan_modes >>  0) & 0xff);
-		msgb_put_u8(msg, (trx->support.chan_modes >>  8) & 0xff);
-		msgb_put_u8(msg, (trx->support.chan_modes >> 16) & 0xff);
-
-		msgb_tv16_put(msg, NM_IPAC_EIE_RTP_FEATURES, 1); /* TL16 */
-		val = NM_IPAC_F_RTP_FEAT_IR_64k;
-		msgb_put_u8(msg, (val >> 0) & 0xff);
-
-		msgb_tv16_put(msg, NM_IPAC_EIE_RSL_FEATURES, 1); /* TL16 */
-		val = NM_IPAC_F_RSL_FEAT_DYN_PDCH_ACT
-		    | NM_IPAC_F_RSL_FEAT_RTP_PT2;
-		msgb_put_u8(msg, (val >> 0) & 0xff);
-		break;
-	case NM_OC_GPRS_CELL:
-		msgb_tv16_put(msg, NM_IPAC_EIE_GPRS_CODING, 2); /* TL16 */
-		msgb_put_u8(msg, (bts->gprs.cell.support.gprs_codings >> 0) & 0xff);
-		msgb_put_u8(msg, (bts->gprs.cell.support.gprs_codings >> 8) & 0xff);
-		break;
-	default:
-		msgb_get(msg, 1 + 2); /* TL16 */
-		return -ENOTSUP;
-	}
+	phy_version = pinst && strlen(pinst->version) ? pinst->version : "Unknown";
+	total_len = abis_nm_put_sw_file(msg, btsatttr2str(TRX_PHY_VERSION), phy_version, true);
 
 	/* Finally, update the length */
-	osmo_store16be((uint16_t)(msg->tail - (len + 2)), len);
+	osmo_store16be(total_len, len);
+}
+
+/* Handle a list of attributes requested by the BSC, compose
+ * TRX-specific Get Attribute Response IE as per 9.4.64. */
+static inline int handle_attrs_trx(struct msgb *out_msg, const struct gsm_bts_trx *trx,
+				   const uint8_t *attr, uint16_t attr_len)
+{
+	uint8_t num_unsupported = 0;
+	uint8_t *buf;
+	int i;
+
+	if (!trx) {
+		LOGP(DOML, LOGL_ERROR, "%s: O&M Get Attributes for unknown TRX\n", gsm_trx_name(trx));
+		return -NM_NACK_TRXNR_UNKN;
+	}
+
+	for (i = 0; i < attr_len; i++) {
+		switch (attr[i]) {
+		case NM_ATT_SW_CONFIG:
+			add_trx_attr(out_msg, trx);
+			break;
+		default:
+			LOGP(DOML, LOGL_ERROR, "%s: O&M Get Attributes [%u], %s is unsupported by TRX\n",
+			     gsm_trx_name(trx), i, get_value_string(abis_nm_att_names, attr[i]));
+			/* Push this tag to the list of unsupported attributes */
+			buf = msgb_push(out_msg, 1);
+			*buf = attr[i];
+			num_unsupported++;
+		}
+	}
+
+	/* Push the amount of unsupported attributes */
+	buf = msgb_push(out_msg, 1);
+	*buf = num_unsupported;
+
+	return 0;
+}
+
+/* Handle a list of attributes requested by the BSC, compose
+ * BTS-specific Get Attribute Response IE as per 9.4.64. */
+static inline int handle_attrs_bts(struct msgb *out_msg, const struct gsm_bts *bts,
+				   const uint8_t *attr, uint16_t attr_len)
+{
+	uint8_t num_unsupported = 0;
+	uint8_t *buf;
+	int i;
+
+	if (!bts) {
+		LOGP(DOML, LOGL_ERROR, "O&M Get Attributes for unknown BTS\n");
+		return -NM_NACK_BTSNR_UNKN;
+	}
+
+	for (i = 0; i < attr_len; i++) {
+		switch (attr[i]) {
+		case NM_ATT_SW_CONFIG:
+			add_bts_attrs(out_msg, bts);
+			break;
+		case NM_ATT_MANUF_ID:
+			add_bts_feat(out_msg, bts);
+			break;
+		default:
+			LOGP(DOML, LOGL_ERROR, "O&M Get Attributes [%u], %s is unsupported by BTS\n", i,
+			     get_value_string(abis_nm_att_names, attr[i]));
+			/* Push this tag to the list of unsupported attributes */
+			buf = msgb_push(out_msg, 1);
+			*buf = attr[i];
+			num_unsupported++;
+		}
+	}
+
+	/* Push the amount of unsupported attributes */
+	buf = msgb_push(out_msg, 1);
+	*buf = num_unsupported;
 
 	return 0;
 }
@@ -284,52 +295,31 @@ static int oml_tx_attr_resp(const struct gsm_abis_mo *mo,
 			    const uint8_t *attr, uint16_t attr_len)
 {
 	struct msgb *nmsg = oml_msgb_alloc();
-	unsigned int num_unsupported = 0;
+	const char *mo_name = gsm_abis_mo_name(mo);
 	int rc;
 
 	if (!nmsg)
 		return -NM_NACK_CANT_PERFORM;
 
-	for (unsigned int i = 0; i < attr_len; i++) {
-		switch (attr[i]) {
-		case NM_ATT_OPER_STATE:
-			msgb_tv16_put(nmsg, attr[i], 1);
-			msgb_put_u8(nmsg, mo->nm_state.operational);
-			break;
-		case NM_ATT_ADM_STATE:
-			msgb_tv16_put(nmsg, attr[i], 1);
-			msgb_put_u8(nmsg, mo->nm_state.administrative);
-			break;
-		case NM_ATT_AVAIL_STATUS:
-			msgb_tv16_put(nmsg, attr[i], 1);
-			msgb_put_u8(nmsg, mo->nm_state.availability);
-			break;
-		case NM_ATT_SW_CONFIG:
-			if (add_att_sw_config(nmsg, mo) != 0)
-				goto unsupported;
-			break;
-		case NM_ATT_MANUF_ID:
-			if (mo->obj_class == NM_OC_BTS)
-				add_bts_feat(nmsg, mo->bts);
-			else
-				goto unsupported;
-			break;
-		case NM_ATT_IPACC_SUPP_FEATURES:
-			if (add_att_ipacc_features(nmsg, mo) != 0)
-				goto unsupported;
-			break;
-		default:
-unsupported:
-			LOGP(DOML, LOGL_ERROR, "%s: O&M Get Attributes [%u], %s is unsupported\n",
-			     gsm_abis_mo_name(mo), i, get_value_string(abis_nm_att_names, attr[i]));
-			/* Push this tag to the list of unsupported attributes */
-			msgb_push_u8(nmsg, attr[i]);
-			num_unsupported++;
-		}
+	switch (mo->obj_class) {
+	case NM_OC_BTS:
+		rc = handle_attrs_bts(nmsg, mo->bts, attr, attr_len);
+		break;
+	case NM_OC_BASEB_TRANSC:
+		rc = handle_attrs_trx(nmsg, gsm_bts_trx_num(mo->bts, mo->obj_inst.trx_nr), attr, attr_len);
+		break;
+	default:
+		LOGP(DOML, LOGL_ERROR, "%s: Unsupported MO class in Get Attribute Response\n",
+		     mo_name);
+		rc = -NM_NACK_OBJCLASS_NOTSUPP;
 	}
 
-	/* Push the amount of unsupported attributes */
-	msgb_push_u8(nmsg, num_unsupported);
+	if (rc < 0) {
+		LOGP(DOML, LOGL_ERROR, "%s: Tx Get Attribute Response FAILED with rc=%d\n",
+		     mo_name, rc);
+		msgb_free(nmsg);
+		return rc;
+	}
 
 	/* Push Get Attribute Response Info TL (actually TV where V is L) */
 	msgb_tv16_push(nmsg, NM_ATT_GET_ARI, msgb_length(nmsg));
@@ -1544,7 +1534,6 @@ static int rx_oml_ipa_rsl_connect(struct gsm_bts *bts, struct msgb *msg,
 	struct e1inp_sign_link *oml_link = bts->oml_link;
 	const struct abis_om_fom_hdr *foh = msgb_l3(msg);
 	struct gsm_bts_trx *trx = gsm_bts_trx_num(bts, foh->obj_inst.trx_nr);
-	struct gsm_bts_bb_trx *bb_transc;
 	const char *trx_name;
 	struct in_addr in;
 	uint16_t port = IPA_TCP_PORT_RSL;
@@ -1558,7 +1547,6 @@ static int rx_oml_ipa_rsl_connect(struct gsm_bts *bts, struct msgb *msg,
 
 	if (TLVP_PRESENT(tp, NM_ATT_IPACC_DST_IP_PORT))
 		port = ntohs(tlvp_val16_unal(tp, NM_ATT_IPACC_DST_IP_PORT));
-
 	if (TLVP_PRESENT(tp, NM_ATT_IPACC_STREAM_ID))
 		stream_id = *TLVP_VAL(tp, NM_ATT_IPACC_STREAM_ID);
 
@@ -1569,16 +1557,22 @@ static int rx_oml_ipa_rsl_connect(struct gsm_bts *bts, struct msgb *msg,
 		goto tx_ack_nack;
 	}
 
-	bb_transc = &trx->bb_transc;
-	osmo_sockaddr_str_from_in_addr(&bb_transc->rsl.rem_addrstr, &in, port);
-	bb_transc->rsl.tei = stream_id;
-
 	trx_name = gsm_trx_name(trx);
-	LOGP(DOML, LOGL_INFO, "%s: Rx IPA RSL CONNECT IP=%s PORT=%u STREAM=0x%02x\n",
-	     trx_name, bb_transc->rsl.rem_addrstr.ip, bb_transc->rsl.rem_addrstr.port,
-	     bb_transc->rsl.tei);
 
-	rc = 0;
+	LOGP(DOML, LOGL_INFO, "%s: Rx IPA RSL CONNECT IP=%s PORT=%u STREAM=0x%02x\n",
+	     trx_name, inet_ntoa(in), port, stream_id);
+
+	if (bts->variant == BTS_OSMO_OMLDUMMY) {
+		rc = 0;
+		LOGP(DOML, LOGL_NOTICE, "%s: Not connecting RSL in OML-DUMMY!\n", trx_name);
+	} else {
+		trx->rsl_tei = stream_id;
+		rc = e1inp_ipa_bts_rsl_connect_n(oml_link->ts->line, inet_ntoa(in), port, trx->nr);
+		if (rc < 0) {
+			LOGP(DOML, LOGL_NOTICE, "%s: Error connecting IPA RSL: %d\n", trx_name, rc);
+			rc = NM_NACK_CANT_PERFORM;
+		}
+	}
 
 tx_ack_nack:
 	/* The ACK/NACK is expected to contain all IEs */
