@@ -43,13 +43,14 @@
 #include <osmo-bts/paging.h>
 #include <osmo-bts/signal.h>
 #include <osmo-bts/pcu_if.h>
+#include <osmo-bts/notification.h>
 
 #define MAX_PAGING_BLOCKS_CCCH	9
 #define MAX_BS_PA_MFRMS		9
 
 enum paging_record_type {
-	PAGING_RECORD_PAGING,
-	PAGING_RECORD_IMM_ASS
+	PAGING_RECORD_NORMAL,
+	PAGING_RECORD_MACBLOCK
 };
 
 struct paging_record {
@@ -60,10 +61,12 @@ struct paging_record {
 			time_t expiration_time;
 			uint8_t chan_needed;
 			uint8_t identity_lv[9];
-		} paging;
+		} normal;
 		struct {
 			uint8_t msg[GSM_MACBLOCK_LEN];
-		} imm_ass;
+			bool confirm;
+			uint32_t msg_id; /* used as identifier for confirmation */
+		} macblock;
 	} u;
 };
 
@@ -88,9 +91,9 @@ struct paging_state {
 
 /* The prioritization of cs pagings is controlled by a hysteresis. When the
  * fill state of the paging queue exceeds the upper fill level
- * THRESHOLD_CONGESTED [%], then PS pagings (immediate assignments) will be
- * dropped until fill state of the paging queue drops under the lower fill
- * level THRESHOLD_CLEAR [%]. */
+ * THRESHOLD_CONGESTED [%], then PS pagings (immediate assignments and pagings
+ * from the PCU) will be dropped until fill state of the paging queue drops
+ * under the lower fill level THRESHOLD_CLEAR [%]. */
 #define THRESHOLD_CONGESTED 66 /* (percent of num_paging_max) */
 #define THRESHOLD_CLEAR 50 /* (percent of num_paging_max) */
 
@@ -230,13 +233,13 @@ int paging_add_identity(struct paging_state *ps, uint8_t paging_group,
 
 	/* Check if we already have this identity */
 	llist_for_each_entry(pr, group_q, list) {
-		if (pr->type != PAGING_RECORD_PAGING)
+		if (pr->type != PAGING_RECORD_NORMAL)
 			continue;
-		if (identity_lv[0] == pr->u.paging.identity_lv[0] &&
-		    !memcmp(identity_lv+1, pr->u.paging.identity_lv+1,
+		if (identity_lv[0] == pr->u.normal.identity_lv[0] &&
+		    !memcmp(identity_lv+1, pr->u.normal.identity_lv+1,
 							identity_lv[0])) {
 			LOGP(DPAG, LOGL_INFO, "Ignoring duplicate paging\n");
-			pr->u.paging.expiration_time =
+			pr->u.normal.expiration_time =
 					time(NULL) + ps->paging_lifetime;
 			return -EEXIST;
 		}
@@ -245,9 +248,9 @@ int paging_add_identity(struct paging_state *ps, uint8_t paging_group,
 	pr = talloc_zero(ps, struct paging_record);
 	if (!pr)
 		return -ENOMEM;
-	pr->type = PAGING_RECORD_PAGING;
+	pr->type = PAGING_RECORD_NORMAL;
 
-	if (*identity_lv + 1 > sizeof(pr->u.paging.identity_lv)) {
+	if (*identity_lv + 1 > sizeof(pr->u.normal.identity_lv)) {
 		talloc_free(pr);
 		return -E2BIG;
 	}
@@ -255,9 +258,9 @@ int paging_add_identity(struct paging_state *ps, uint8_t paging_group,
 	LOGP(DPAG, LOGL_INFO, "Add paging to queue (group=%u, queue_len=%u)\n",
 		paging_group, ps->num_paging+1);
 
-	pr->u.paging.expiration_time = time(NULL) + ps->paging_lifetime;
-	pr->u.paging.chan_needed = chan_needed;
-	memcpy(&pr->u.paging.identity_lv, identity_lv, identity_lv[0]+1);
+	pr->u.normal.expiration_time = time(NULL) + ps->paging_lifetime;
+	pr->u.normal.chan_needed = chan_needed;
+	memcpy(&pr->u.normal.identity_lv, identity_lv, identity_lv[0]+1);
 
 	/* enqueue the new identity to the HEAD of the queue,
 	 * to ensure it will be paged quickly at least once.  */
@@ -267,44 +270,59 @@ int paging_add_identity(struct paging_state *ps, uint8_t paging_group,
 	return 0;
 }
 
-/* Add an IMM.ASS message to the paging queue */
-int paging_add_imm_ass(struct paging_state *ps, const uint8_t *data,
-                       uint8_t len, bool from_pcu)
+/* Convert the last three digits of a given IMSI string to their decimal representation. In case the given IMSI string
+ * is shorter than three or zero digits, it will be assumed as "000" */
+static uint16_t convert_imsi_to_decimal(const char *imsi)
+{
+	uint16_t _imsi;
+	size_t imsi_len = strlen(imsi);
+
+	/* Tha paging group is calculated from the last three digits of the IMSI */
+	if (imsi_len < 3) {
+		LOGP(DPAG, LOGL_ERROR, "short IMSI (%zu digits), will assume \"000\" to calculate paging group\n", imsi_len);
+		_imsi = 0;
+	} else {
+		imsi = imsi + imsi_len - 3;
+		_imsi = 100 * ((*(imsi++)) - '0');
+		_imsi += 10 * ((*(imsi++)) - '0');
+		_imsi += (*(imsi++)) - '0';
+	}
+
+	return _imsi;
+}
+
+/* Add a ready formatted MAC block message to the paging queue, this can be an IMMEDIATE ASSIGNMENT, or a
+ * PAGING COMMAND (from the PCU) */
+int paging_add_macblock(struct paging_state *ps, uint32_t msg_id, const char *imsi, bool confirm, const uint8_t *macblock)
 {
 	struct llist_head *group_q;
 	struct paging_record *pr;
-	uint16_t imsi, paging_group;
+	uint16_t paging_group;
+	uint16_t _imsi;
 
 	check_congestion(ps);
 
-	if (ps->cs_priority_active && from_pcu) {
+	if (ps->cs_priority_active) {
 		LOGP(DPAG, LOGL_NOTICE, "Dropping paging for PS, queue congested (%u)\n",
 			ps->num_paging);
 		rate_ctr_inc2(ps->bts->ctrs, BTS_CTR_PAGING_DROP_PS);
 		return -ENOSPC;
 	}
 
-	if (len != GSM_MACBLOCK_LEN + 3) {
-		LOGP(DPAG, LOGL_ERROR, "IMM.ASS invalid length %d\n", len);
-		return -EINVAL;
-	}
-	len -= 3;
-
-	imsi = 100 * ((*(data++)) - '0');
-	imsi += 10 * ((*(data++)) - '0');
-	imsi += (*(data++)) - '0';
-	paging_group = gsm0502_calc_paging_group(&ps->chan_desc, imsi);
-
+	_imsi = convert_imsi_to_decimal(imsi);
+	paging_group = gsm0502_calc_paging_group(&ps->chan_desc, _imsi);
 	group_q = &ps->paging_queue[paging_group];
 
 	pr = talloc_zero(ps, struct paging_record);
 	if (!pr)
 		return -ENOMEM;
-	pr->type = PAGING_RECORD_IMM_ASS;
+	pr->type = PAGING_RECORD_MACBLOCK;
 
-	LOGP(DPAG, LOGL_INFO, "Add IMM.ASS to queue (group=%u)\n",
+	LOGP(DPAG, LOGL_INFO, "Add MAC block to paging queue (group=%u)\n",
 		paging_group);
-	memcpy(pr->u.imm_ass.msg, data, GSM_MACBLOCK_LEN);
+	memcpy(pr->u.macblock.msg, macblock, GSM_MACBLOCK_LEN);
+	pr->u.macblock.confirm = confirm;
+	pr->u.macblock.msg_id = msg_id;
 
 	/* enqueue the new message to the HEAD of the queue */
 	llist_add(&pr->list, group_q);
@@ -314,22 +332,6 @@ int paging_add_imm_ass(struct paging_state *ps, const uint8_t *data,
 
 #define L2_PLEN(len)	(((len - 1) << 2) | 0x01)
 
-/* abstract representation of P1 rest octets; we only implement those parts we need for now */
-struct p1_rest_octets {
-	bool packet_page_ind[2];
-	bool r8_present;
-	struct {
-		bool prio_ul_access;
-		bool etws_present;
-		struct {
-			bool is_first;
-			uint8_t page_nr;
-			const uint8_t *page;
-			size_t page_bytes;
-		} etws;
-	} r8;
-};
-
 /* 3GPP TS 44.018 10.5.2.23 append a segment/page of an ETWS primary notification to given bitvec */
 static void append_etws_prim_notif(struct bitvec *bv, bool is_first, uint8_t page_nr,
 				   const uint8_t *etws, ssize_t etws_len)
@@ -356,13 +358,27 @@ static void append_etws_prim_notif(struct bitvec *bv, bool is_first, uint8_t pag
 }
 
 /* 3GPP TS 44.018 10.5.2.23 append P1 Rest Octets to given bit-vector */
-static void append_p1_rest_octets(struct bitvec *bv, const struct p1_rest_octets *p1ro)
+void append_p1_rest_octets(struct bitvec *bv, const struct p1_rest_octets *p1ro,
+			   const struct asci_notification *notif)
 {
 	/* Paging 1 RO (at least 10 bits before ETWS struct) */
-	bitvec_set_bit(bv, L);		/* no NLN */
+	if (p1ro->nln_pch.present) {
+		bitvec_set_bit(bv, H);
+		bitvec_set_uint(bv, p1ro->nln_pch.nln, 2);
+		bitvec_set_uint(bv, p1ro->nln_pch.nln_status, 1);
+	} else {
+		bitvec_set_bit(bv, L);		/* no NLN */
+	}
 	bitvec_set_bit(bv, L);		/* no Priority1 */
 	bitvec_set_bit(bv, L);		/* no Priority2 */
-	bitvec_set_bit(bv, L);		/* no Group Call Info */
+	if (notif) {
+		bitvec_set_bit(bv, H);		/* Group Call Info */
+		append_group_call_information(bv, notif->group_call_ref,
+					      notif->chan_desc.present ? notif->chan_desc.value : NULL,
+					      notif->chan_desc.len);
+	} else {
+		bitvec_set_bit(bv, L);		/* no Group Call Info */
+	}
 	if (p1ro->packet_page_ind[0])
 		bitvec_set_bit(bv, H);		/* Packet Page Indication 1 */
 	else
@@ -387,9 +403,53 @@ static void append_p1_rest_octets(struct bitvec *bv, const struct p1_rest_octets
 	}
 }
 
+/* 3GPP TS 44.018 10.5.2.24 append P2 Rest Octets to given bit-vector */
+void append_p2_rest_octets(struct bitvec *bv, const struct p2_rest_octets *p2ro)
+{
+	/* {L | H <CN3: bit (2)>} */
+	if (p2ro->cneed.present) {
+		bitvec_set_bit(bv, H);
+		bitvec_set_uint(bv, p2ro->cneed.cn3, 2);
+	} else
+		bitvec_set_bit(bv, L);		/* no CN3 */
+
+	/* {L | H < NLN(PCH) : bit (2) <NLN status(PCH) : bit>} */
+	if (p2ro->nln_pch.present) {
+		bitvec_set_bit(bv, H);
+		bitvec_set_uint(bv, p2ro->nln_pch.nln, 2);
+		bitvec_set_uint(bv, p2ro->nln_pch.nln_status, 1);
+	} else
+		bitvec_set_bit(bv, L);		/* no NLN */
+
+	/* Note: If this needs to be extended in the future, check if it actually fits into rest of P2! */
+}
+
+/* 3GPP TS 44.018 10.5.2.25 append P3 Rest Octets to given bit-vector */
+void append_p3_rest_octets(struct bitvec *bv, const struct p3_rest_octets *p3ro)
+{
+	/* {L | H <CN3: bit (2)> <CN3: bit (2)>} */
+	if (p3ro->cneed.present) {
+		bitvec_set_bit(bv, H);
+		bitvec_set_uint(bv, p3ro->cneed.cn3, 2);
+		bitvec_set_uint(bv, p3ro->cneed.cn4, 2);
+	} else
+		bitvec_set_bit(bv, L);		/* no CN3/CN4 */
+
+	/* {L | H < NLN(PCH) : bit (2) <NLN status(PCH) : bit>} */
+	if (p3ro->nln_pch.present) {
+		bitvec_set_bit(bv, H);
+		bitvec_set_uint(bv, p3ro->nln_pch.nln, 2);
+		bitvec_set_uint(bv, p3ro->nln_pch.nln_status, 1);
+	} else
+		bitvec_set_bit(bv, L);		/* no NLN */
+
+	/* Note: If this needs to be extended in the future, check if it actually fits into 3 octets! */
+}
+
 static int fill_paging_type_1(uint8_t *out_buf, const uint8_t *identity1_lv,
 				uint8_t chan1, const uint8_t *identity2_lv,
-				uint8_t chan2, const struct p1_rest_octets *p1ro)
+				uint8_t chan2, const struct p1_rest_octets *p1ro,
+				const struct asci_notification *notif)
 {
 	struct gsm48_paging1 *pt1 = (struct gsm48_paging1 *) out_buf;
 	unsigned int ro_len;
@@ -420,7 +480,7 @@ static int fill_paging_type_1(uint8_t *out_buf, const uint8_t *identity1_lv,
 			.data = cur,
 		};
 
-		append_p1_rest_octets(&bv, p1ro);
+		append_p1_rest_octets(&bv, p1ro, notif);
 	}
 
 	return GSM_MACBLOCK_LEN;
@@ -428,10 +488,12 @@ static int fill_paging_type_1(uint8_t *out_buf, const uint8_t *identity1_lv,
 
 static int fill_paging_type_2(uint8_t *out_buf, const uint8_t *tmsi1_lv,
 				uint8_t cneed1, const uint8_t *tmsi2_lv,
-				uint8_t cneed2, const uint8_t *identity3_lv)
+				uint8_t cneed2, const uint8_t *identity3_lv,
+				const struct p2_rest_octets *p2ro)
 {
 	struct gsm48_paging2 *pt2 = (struct gsm48_paging2 *) out_buf;
 	uint32_t tmsi;
+	unsigned int ro_len;
 	uint8_t *cur;
 	int rc;
 
@@ -455,16 +517,32 @@ static int fill_paging_type_2(uint8_t *out_buf, const uint8_t *tmsi1_lv,
 
 	pt2->l2_plen = L2_PLEN(cur - out_buf);
 
-	return cur - out_buf;
+	/* Pad remaining octets with constant '2B'O */
+	ro_len = GSM_MACBLOCK_LEN - (cur - out_buf);
+	memset(cur, GSM_MACBLOCK_PADDING, ro_len);
+
+	/* Optional P2 Rest Octets */
+	if (p2ro) {
+		struct bitvec bv = {
+			.data_len = ro_len,
+			.data = cur,
+		};
+
+		append_p2_rest_octets(&bv, p2ro);
+	}
+
+	return GSM_MACBLOCK_LEN;
 }
 
 static int fill_paging_type_3(uint8_t *out_buf, const uint8_t *tmsi1_lv, uint8_t cneed1,
 				const uint8_t *tmsi2_lv, uint8_t cneed2,
-				const uint8_t *tmsi3_lv, uint8_t cneed3,
-				const uint8_t *tmsi4_lv, uint8_t cneed4)
+				const uint8_t *tmsi3_lv, const uint8_t *tmsi4_lv,
+				const struct p3_rest_octets *p3ro)
 {
 	struct gsm48_paging3 *pt3 = (struct gsm48_paging3 *) out_buf;
 	uint32_t tmsi;
+	unsigned int ro_len;
+	uint8_t *cur;
 	int rc;
 
 	memset(out_buf, 0, sizeof(*pt3));
@@ -486,13 +564,25 @@ static int fill_paging_type_3(uint8_t *out_buf, const uint8_t *tmsi1_lv, uint8_t
 	rc = tmsi_mi_to_uint(&tmsi, tmsi4_lv);
 	if (rc == 0)
 		pt3->tmsi4 = tmsi;
+	cur = out_buf + 20;	/* Cannot use sizeof(*pt3), because it has more octets. */
 
-	/* The structure definition in libosmocore is wrong. It includes as last
-	 * byte some invalid definition of chneed3/chneed4, so we must do this by hand
-	 * here and cannot rely on sizeof(*pt3) */
-	out_buf[20] = (0x23 & ~0xf8) | 0x80 | (cneed3 & 3) << 5 | (cneed4 & 3) << 3;
+	pt3->l2_plen = L2_PLEN(cur - out_buf);
 
-	return 21;
+	/* Pad remaining octets with constant '2B'O */
+	ro_len = GSM_MACBLOCK_LEN - (cur - out_buf);
+	memset(cur, GSM_MACBLOCK_PADDING, ro_len);
+
+	/* Optional P3 Rest Octets */
+	if (p3ro) {
+		struct bitvec bv = {
+			.data_len = ro_len,
+			.data = cur,
+		};
+
+		append_p3_rest_octets(&bv, p3ro);
+	}
+
+	return GSM_MACBLOCK_LEN;
 }
 
 static const uint8_t empty_id_lv[] = { 0x01, 0xF0 };
@@ -509,7 +599,7 @@ static struct paging_record *dequeue_pr(struct llist_head *group_q)
 
 static int pr_is_imsi(struct paging_record *pr)
 {
-	if ((pr->u.paging.identity_lv[1] & 7) == GSM_MI_TYPE_IMSI)
+	if ((pr->u.normal.identity_lv[1] & 7) == GSM_MI_TYPE_IMSI)
 		return 1;
 	else
 		return 0;
@@ -538,6 +628,7 @@ static void sort_pr_tmsi_imsi(struct paging_record *pr[], unsigned int n)
 static void build_p1_rest_octets(struct p1_rest_octets *p1ro, struct gsm_bts *bts)
 {
 	memset(p1ro, 0, sizeof(*p1ro));
+	p1ro->nln_pch.present = false;
 	p1ro->packet_page_ind[0] = false;
 	p1ro->packet_page_ind[1] = false;
 	p1ro->r8_present = true;
@@ -590,16 +681,24 @@ int paging_gen_msg(struct paging_state *ps, uint8_t *out_buf, struct gsm_time *g
 	if (ps->bts->etws.prim_notif) {
 		struct p1_rest_octets p1ro;
 		build_p1_rest_octets(&p1ro, bts);
-		len = fill_paging_type_1(out_buf, empty_id_lv, 0, NULL, 0, &p1ro);
+		/* we intentioanally don't try to add notifications here, as ETWS is more critical */
+		len = fill_paging_type_1(out_buf, empty_id_lv, 0, NULL, 0, &p1ro, NULL);
 	} else if (llist_empty(group_q)) {
+		struct p1_rest_octets p1ro;
+		memset(&p1ro, 0, sizeof(p1ro));
+		/* Use NLN to notify MS about ongoing VGCS/VBS calls.
+		 * This is required to make the phone read the NCH to get an updated list of ongoing calls.
+		 * Without this the phone will not allow making VGCS/VBS calls. */
+		p1ro.nln_pch.present = (bts->asci.pos_nch >= 0);
+		p1ro.nln_pch.nln = bts->asci.nln;
+		p1ro.nln_pch.nln_status = bts->asci.nln_status;
 		/* There is nobody to be paged, send Type1 with two empty ID */
 		//DEBUGP(DPAG, "Tx PAGING TYPE 1 (empty)\n");
-		len = fill_paging_type_1(out_buf, empty_id_lv, 0,
-					 NULL, 0, NULL);
+		len = fill_paging_type_1(out_buf, empty_id_lv, 0, NULL, 0, &p1ro, NULL);
 		*is_empty = 1;
 	} else {
 		struct paging_record *pr[4];
-		unsigned int num_pr = 0, imm_ass = 0;
+		unsigned int num_pr = 0, macblock = 0;
 		time_t now = time(NULL);
 		unsigned int i, num_imsi = 0;
 
@@ -611,9 +710,9 @@ int paging_gen_msg(struct paging_state *ps, uint8_t *out_buf, struct gsm_time *g
 				break;
 			pr[i] = dequeue_pr(group_q);
 
-			/* check for IMM.ASS */
-			if (pr[i]->type == PAGING_RECORD_IMM_ASS) {
-				imm_ass = 1;
+			/* check for MAC block */
+			if (pr[i]->type == PAGING_RECORD_MACBLOCK) {
+				macblock = 1;
 				break;
 			}
 
@@ -624,17 +723,18 @@ int paging_gen_msg(struct paging_state *ps, uint8_t *out_buf, struct gsm_time *g
 				num_imsi++;
 		}
 
-		/* if we have an IMMEDIATE ASSIGNMENT */
-		if (imm_ass) {
-			/* re-add paging records */
+		/* Handle MAC block (from the PCU) */
+		if (macblock) {
+			/* re-add normal paging records */
 			for (i = 0; i < num_pr; i++)
 				llist_add(&pr[i]->list, group_q);
 
-			/* get message and free record */
-			memcpy(out_buf, pr[num_pr]->u.imm_ass.msg,
-							GSM_MACBLOCK_LEN);
-			pcu_tx_pch_data_cnf(gt->fn, pr[num_pr]->u.imm_ass.msg,
+			/* get MAC block message and free record */
+			memcpy(out_buf, pr[num_pr]->u.macblock.msg,
 							GSM_MACBLOCK_LEN);
+			/* send a confirmation back (if required) */
+			if (pr[num_pr]->u.macblock.confirm)
+				pcu_tx_data_cnf(pr[num_pr]->u.macblock.msg_id, PCU_IF_SAPI_PCH_2);
 			talloc_free(pr[num_pr]);
 			return GSM_MACBLOCK_LEN;
 		}
@@ -645,24 +745,39 @@ int paging_gen_msg(struct paging_state *ps, uint8_t *out_buf, struct gsm_time *g
 		if (num_pr == 4 && num_imsi == 0) {
 			/* No IMSI: easy case, can use TYPE 3 */
 			DEBUGP(DPAG, "Tx PAGING TYPE 3 (4 TMSI)\n");
+			struct p3_rest_octets p3ro;
+			memset(&p3ro, 0, sizeof(p3ro));
+			p3ro.cneed.present = true;
+			p3ro.cneed.cn3 = pr[2]->u.normal.chan_needed;
+			p3ro.cneed.cn4 = pr[3]->u.normal.chan_needed;
+			p3ro.nln_pch.present = (bts->asci.pos_nch >= 0);
+			p3ro.nln_pch.nln = bts->asci.nln;
+			p3ro.nln_pch.nln_status = bts->asci.nln_status;
 			len = fill_paging_type_3(out_buf,
-						 pr[0]->u.paging.identity_lv,
-						 pr[0]->u.paging.chan_needed,
-						 pr[1]->u.paging.identity_lv,
-						 pr[1]->u.paging.chan_needed,
-						 pr[2]->u.paging.identity_lv,
-						 pr[2]->u.paging.chan_needed,
-						 pr[3]->u.paging.identity_lv,
-						 pr[3]->u.paging.chan_needed);
+						 pr[0]->u.normal.identity_lv,
+						 pr[0]->u.normal.chan_needed,
+						 pr[1]->u.normal.identity_lv,
+						 pr[1]->u.normal.chan_needed,
+						 pr[2]->u.normal.identity_lv,
+						 pr[3]->u.normal.identity_lv,
+						 &p3ro);
 		} else if (num_pr >= 3 && num_imsi <= 1) {
 			/* 3 or 4, of which only up to 1 is IMSI */
 			DEBUGP(DPAG, "Tx PAGING TYPE 2 (2 TMSI,1 xMSI)\n");
+			struct p2_rest_octets p2ro;
+			memset(&p2ro, 0, sizeof(p2ro));
+			p2ro.cneed.present = true;
+			p2ro.cneed.cn3 = pr[2]->u.normal.chan_needed;
+			p2ro.nln_pch.present = (bts->asci.pos_nch >= 0);
+			p2ro.nln_pch.nln = bts->asci.nln;
+			p2ro.nln_pch.nln_status = bts->asci.nln_status;
 			len = fill_paging_type_2(out_buf,
-						 pr[0]->u.paging.identity_lv,
-						 pr[0]->u.paging.chan_needed,
-						 pr[1]->u.paging.identity_lv,
-						 pr[1]->u.paging.chan_needed,
-						 pr[2]->u.paging.identity_lv);
+						 pr[0]->u.normal.identity_lv,
+						 pr[0]->u.normal.chan_needed,
+						 pr[1]->u.normal.identity_lv,
+						 pr[1]->u.normal.chan_needed,
+						 pr[2]->u.normal.identity_lv,
+						 &p2ro);
 			if (num_pr == 4) {
 				/* re-add #4 for next time */
 				llist_add(&pr[3]->list, group_q);
@@ -670,19 +785,21 @@ int paging_gen_msg(struct paging_state *ps, uint8_t *out_buf, struct gsm_time *g
 			}
 		} else if (num_pr == 1) {
 			DEBUGP(DPAG, "Tx PAGING TYPE 1 (1 xMSI,1 empty)\n");
+			/* TODO: check if we can include an ASCI notification */
 			len = fill_paging_type_1(out_buf,
-						 pr[0]->u.paging.identity_lv,
-						 pr[0]->u.paging.chan_needed,
-						 NULL, 0, NULL);
+						 pr[0]->u.normal.identity_lv,
+						 pr[0]->u.normal.chan_needed,
+						 NULL, 0, NULL, NULL);
 		} else {
 			/* 2 (any type) or
 			 * 3 or 4, of which only 2 will be sent */
 			DEBUGP(DPAG, "Tx PAGING TYPE 1 (2 xMSI)\n");
+			/* TODO: check if we can include an ASCI notification */
 			len = fill_paging_type_1(out_buf,
-						 pr[0]->u.paging.identity_lv,
-						 pr[0]->u.paging.chan_needed,
-						 pr[1]->u.paging.identity_lv,
-						 pr[1]->u.paging.chan_needed, NULL);
+						 pr[0]->u.normal.identity_lv,
+						 pr[0]->u.normal.chan_needed,
+						 pr[1]->u.normal.identity_lv,
+						 pr[1]->u.normal.chan_needed, NULL, NULL);
 			if (num_pr >= 3) {
 				/* re-add #4 for next time */
 				llist_add(&pr[2]->list, group_q);
@@ -702,7 +819,7 @@ int paging_gen_msg(struct paging_state *ps, uint8_t *out_buf, struct gsm_time *g
 			rate_ctr_inc2(bts->ctrs, BTS_CTR_PAGING_SENT);
 			/* check if we can expire the paging record,
 			 * or if we need to re-queue it */
-			if (pr[i]->u.paging.expiration_time <= now) {
+			if (pr[i]->u.normal.expiration_time <= now) {
 				talloc_free(pr[i]);
 				ps->num_paging--;
 				LOGP(DPAG, LOGL_INFO, "Removed paging record, queue_len=%u\n",
