@@ -140,6 +140,20 @@ int bts_vty_go_parent(struct vty *vty)
 	return vty->node;
 }
 
+int bts_vty_is_config_node(struct vty *vty, int node)
+{
+	switch (node) {
+	case TRX_NODE:
+	case BTS_NODE:
+	case PHY_NODE:
+	case PHY_INST_NODE:
+	case OSMUX_NODE:
+		return 1;
+	default:
+		return 0;
+	}
+}
+
 static const char osmobts_copyright[] =
 	"Copyright (C) 2010-2011 by Harald Welte, Andreas Eversberg and On-Waves\r\n"
 	"Copyright (C) 2011-2022 by sysmocom - s.f.m.c. GmbH\r\n"
@@ -152,6 +166,7 @@ struct vty_app_info bts_vty_info = {
 	.version	= PACKAGE_VERSION,
 	.copyright	= osmobts_copyright,
 	.go_parent_cb	= bts_vty_go_parent,
+	.is_config_node	= bts_vty_is_config_node,
 	.usr_attr_desc	= {
 		[BTS_VTY_ATTR_NEW_LCHAN] = \
 			"This command applies for newly created lchans",
@@ -1644,10 +1659,10 @@ static void trx_dump_vty(struct vty *vty, const struct gsm_bts_trx *trx)
 
 	vty_out(vty, "  NM State: ");
 	net_dump_nmstate(vty, &trx->mo.nm_state);
-	vty_out(vty, "  RSL State: %s%s", trx->bb_transc.rsl.link ? "connected" : "disconnected", VTY_NEWLINE);
+	vty_out(vty, "  RSL State: %s%s", trx->rsl_link? "connected" : "disconnected", VTY_NEWLINE);
 	vty_out(vty, "  Baseband Transceiver NM State: ");
 	net_dump_nmstate(vty, &trx->bb_transc.mo.nm_state);
-	vty_out(vty, "  IPA stream ID: 0x%02x%s", trx->bb_transc.rsl.tei, VTY_NEWLINE);
+	vty_out(vty, "  IPA stream ID: 0x%02x%s", trx->rsl_tei, VTY_NEWLINE);
 }
 
 static inline void print_all_trx(struct vty *vty, const struct gsm_bts *bts)
