@@ -49,6 +49,8 @@
 #include "lc15bts.h"
 #include "utils.h"
 
+static void dump_lch_par(int logl, GsmL1_LogChParam_t *lch_par, GsmL1_Sapi_t sapi);
+
 static int mph_info_chan_confirm(struct gsm_lchan *lchan,
 			enum osmo_mph_info_type type, uint8_t cause)
 {
@@ -569,8 +571,7 @@ GsmL1_Sapi_t lchan_to_GsmL1_Sapi_t(const struct gsm_lchan *lchan)
 	case GSM_LCHAN_TCH_H:
 		return GsmL1_Sapi_TchH;
 	default:
-		LOGP(DL1C, LOGL_NOTICE, "%s cannot determine L1 SAPI\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_NOTICE, "cannot determine L1 SAPI\n");
 		break;
 	}
 	return GsmL1_Sapi_Idle;
@@ -781,12 +782,8 @@ static void sapi_queue_dispatch(struct gsm_lchan *lchan, int status)
 	talloc_free(cmd);
 
 	if (end || llist_empty(&lchan->sapi_cmds)) {
-		LOGP(DL1C, LOGL_DEBUG,
-			"%s End of SAPI cmd queue encountered.%s\n",
-			gsm_lchan_name(lchan),
-			llist_empty(&lchan->sapi_cmds)
-				? " Queue is now empty."
-				: " More pending.");
+		LOGPLCHAN(lchan, DL1C, LOGL_DEBUG, "End of SAPI cmd queue encountered.%s\n",
+			  llist_empty(&lchan->sapi_cmds) ? " Queue is now empty." : " More pending.");
 		return;
 	}
 
@@ -826,9 +823,8 @@ static int lchan_act_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		goto err;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-ACTIVATE.conf (%s ",
-		gsm_lchan_name(lchan),
-		get_value_string(lc15bts_l1sapi_names, ic->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-ACTIVATE.conf (%s ",
+		  get_value_string(lc15bts_l1sapi_names, ic->sapi));
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(lc15bts_dir_names, ic->dir));
 
@@ -849,19 +845,15 @@ static int lchan_act_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		lchan->sapis_ul[ic->sapi] = status;
 
 	if (llist_empty(&lchan->sapi_cmds)) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Got activation confirmation with empty queue\n",
-				gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Got activation confirmation with empty queue\n");
 		goto err;
 	}
 
 	cmd = llist_entry(lchan->sapi_cmds.next, struct sapi_cmd, entry);
 	if (cmd->sapi != ic->sapi || cmd->dir != ic->dir ||
 			cmd->type != SAPI_CMD_ACTIVATE) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Confirmation mismatch (%d, %d) (%d, %d)\n",
-				gsm_lchan_name(lchan), cmd->sapi, cmd->dir,
-				ic->sapi, ic->dir);
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Confirmation mismatch (%d, %d) (%d, %d)\n",
+			  cmd->sapi, cmd->dir, ic->sapi, ic->dir);
 		goto err;
 	}
 
@@ -942,8 +934,7 @@ static int lchan2lch_par(GsmL1_LogChParam_t *lch_par, struct gsm_lchan *lchan)
 			(struct gsm48_multi_rate_conf *) amr_mrc->gsm48_ie;
 	int j;
 
-	LOGP(DL1C, LOGL_INFO, "%s: %s tch_mode=0x%02x\n",
-		gsm_lchan_name(lchan), __FUNCTION__, lchan->tch_mode);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "%s tch_mode=0x%02x\n", __func__, lchan->tch_mode);
 
 	switch (lchan->tch_mode) {
 	case GSM48_CMODE_SIGN:
@@ -971,7 +962,9 @@ static int lchan2lch_par(GsmL1_LogChParam_t *lch_par, struct gsm_lchan *lchan)
 	case GSM48_CMODE_SPEECH_AMR:
 		lch_par->tch.tchPlType = GsmL1_TchPlType_Amr;
 		set_payload_format(lch_par);
-		lch_par->tch.amrCmiPhase = GsmL1_AmrCmiPhase_Odd; /* FIXME? */
+		/* At call set-up, after every successful handover and after a channel mode modify, the
+		 * default phase (odd) shall be used in downlink direction. */
+		lch_par->tch.amrCmiPhase = GsmL1_AmrCmiPhase_Odd;
 		lch_par->tch.amrInitCodecMode = amr_get_initial_mode(lchan);
 
 		/* initialize to clean state */
@@ -1094,9 +1087,9 @@ static int mph_send_activate_req(struct gsm_lchan *lchan, struct sapi_cmd *cmd)
 		break;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-ACTIVATE.req (hL2=0x%08x, %s ",
-		gsm_lchan_name(lchan), (uint32_t)act_req->hLayer2,
-		get_value_string(lc15bts_l1sapi_names, act_req->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-ACTIVATE.req (hL2=0x%08x, %s ",
+		  (uint32_t)act_req->hLayer2, get_value_string(lc15bts_l1sapi_names, act_req->sapi));
+	dump_lch_par(LOGL_INFO, lch_par, act_req->sapi);
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(lc15bts_dir_names, act_req->dir));
 
@@ -1120,9 +1113,7 @@ static int sapi_activate_cb(struct gsm_lchan *lchan, int status)
 
 	/* FIXME: Error handling */
 	if (status != GsmL1_Status_Success) {
-		LOGP(DL1C, LOGL_ERROR,
-			"%s act failed mark broken due status: %d\n",
-			gsm_lchan_name(lchan), status);
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "act failed mark broken due status: %d\n", status);
 		lchan_set_state(lchan, LCHAN_S_BROKEN);
 		sapi_clear_queue(&lchan->sapi_cmds);
 		mph_info_chan_confirm(lchan, PRIM_INFO_ACTIVATE, RSL_ERR_PROCESSOR_OVERLOAD);
@@ -1169,13 +1160,11 @@ int lchan_activate(struct gsm_lchan *lchan)
 	lchan_set_state(lchan, LCHAN_S_ACT_REQ);
 
 	if (!llist_empty(&lchan->sapi_cmds))
-		LOGP(DL1C, LOGL_ERROR,
-			"%s Trying to activate lchan, but commands in queue\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Trying to activate lchan, but commands in queue\n");
 
 	/* For handover, always start the main channel immediately. lchan->want_dl_sacch_active indicates whether dl
 	 * SACCH should be activated. Also, for HO, start the RACH SAPI. */
-	if (lchan->ho.active == HANDOVER_ENABLED)
+	if (lchan->ho.active == HANDOVER_ENABLED || rsl_chan_rt_is_asci(lchan->rsl_chan_rt))
 		enqueue_sapi_act_cmd(lchan, GsmL1_Sapi_Rach, GsmL1_Dir_RxUplink);
 
 	for (i = 0; i < s4l->num_sapis; i++) {
@@ -1332,9 +1321,8 @@ static int chmod_modif_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		goto err;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-CONFIG.conf (%s) ",
-		gsm_lchan_name(lchan),
-		get_value_string(lc15bts_l1cfgt_names, cc->cfgParamId));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-CONFIG.conf (%s) ",
+		  get_value_string(lc15bts_l1cfgt_names, cc->cfgParamId));
 
 	switch (cc->cfgParamId) {
 	case GsmL1_ConfigParamId_SetLogChParams:
@@ -1366,9 +1354,7 @@ static int chmod_modif_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 			break;
 		}
 		if (llist_empty(&lchan->sapi_cmds)) {
-			LOGP(DL1C, LOGL_ERROR,
-				"%s Got ciphering conf with empty queue\n",
-				gsm_lchan_name(lchan));
+			LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Got ciphering conf with empty queue\n");
 			goto err;
 		}
 
@@ -1419,10 +1405,8 @@ static int mph_send_config_logchpar(struct gsm_lchan *lchan, struct sapi_cmd *cm
 
 	/* FIXME: update encryption */
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-CONFIG.req (%s) ",
-		gsm_lchan_name(lchan),
-		get_value_string(lc15bts_l1sapi_names,
-				 conf_req->cfgParams.setLogChParams.sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-CONFIG.req (%s) ",
+		  get_value_string(lc15bts_l1sapi_names, conf_req->cfgParams.setLogChParams.sapi));
 	LOGPC(DL1C, LOGL_INFO, "cfgParams Tn=%u, subCh=%u, dir=0x%x ",
 			conf_req->cfgParams.setLogChParams.u8Tn,
 			conf_req->cfgParams.setLogChParams.subCh,
@@ -1532,11 +1516,9 @@ static int mph_send_config_ciphering(struct gsm_lchan *lchan, struct sapi_cmd *c
 		return -EINVAL;
 	cfgr->cfgParams.setCipheringParams.cipherId = rsl2l1_ciph[lchan->encr.alg_id];
 
-	LOGP(DL1C, LOGL_NOTICE, "%s SET_CIPHERING (ALG=%u %s)\n",
-		gsm_lchan_name(lchan),
+	LOGPLCHAN(lchan, DL1C, LOGL_NOTICE, "SET_CIPHERING (ALG=%u %s)\n",
 		cfgr->cfgParams.setCipheringParams.cipherId,
-		get_value_string(lc15bts_dir_names,
-				 cfgr->cfgParams.setCipheringParams.dir));
+		get_value_string(lc15bts_dir_names, cfgr->cfgParams.setCipheringParams.dir));
 
 	memcpy(cfgr->cfgParams.setCipheringParams.u8Kc,
 	       lchan->encr.key, lchan->encr.key_len);
@@ -1614,9 +1596,8 @@ static int lchan_deact_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		goto err;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-DEACTIVATE.conf (%s ",
-		gsm_lchan_name(lchan),
-		get_value_string(lc15bts_l1sapi_names, ic->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-DEACTIVATE.conf (%s ",
+		  get_value_string(lc15bts_l1sapi_names, ic->sapi));
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(lc15bts_dir_names, ic->dir));
 
@@ -1638,19 +1619,15 @@ static int lchan_deact_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 
 
 	if (llist_empty(&lchan->sapi_cmds)) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Got de-activation confirmation with empty queue\n",
-				gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Got de-activation confirmation with empty queue\n");
 		goto err;
 	}
 
 	cmd = llist_entry(lchan->sapi_cmds.next, struct sapi_cmd, entry);
 	if (cmd->sapi != ic->sapi || cmd->dir != ic->dir ||
 			cmd->type != SAPI_CMD_DEACTIVATE) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Confirmation mismatch (%d, %d) (%d, %d)\n",
-				gsm_lchan_name(lchan), cmd->sapi, cmd->dir,
-				ic->sapi, ic->dir);
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Confirmation mismatch (%d, %d) (%d, %d)\n",
+			  cmd->sapi, cmd->dir, ic->sapi, ic->dir);
 		goto err;
 	}
 
@@ -1675,9 +1652,8 @@ static int mph_send_deactivate_req(struct gsm_lchan *lchan, struct sapi_cmd *cmd
 	deact_req->sapi = cmd->sapi;
 	deact_req->hLayer3 = (HANDLE)l1if_lchan_to_hLayer(lchan);
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-DEACTIVATE.req (%s ",
-		gsm_lchan_name(lchan),
-		get_value_string(lc15bts_l1sapi_names, deact_req->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-DEACTIVATE.req (%s ",
+		  get_value_string(lc15bts_l1sapi_names, deact_req->sapi));
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(lc15bts_dir_names, deact_req->dir));
 
@@ -1689,8 +1665,7 @@ static int sapi_deactivate_cb(struct gsm_lchan *lchan, int status)
 {
 	/* FIXME: Error handling. There is no NACK... */
 	if (status != GsmL1_Status_Success && lchan->state == LCHAN_S_REL_REQ) {
-		LOGP(DL1C, LOGL_ERROR, "%s is now broken. Stopping the release.\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "is now broken. Stopping the release.\n");
 		lchan_set_state(lchan, LCHAN_S_BROKEN);
 		sapi_clear_queue(&lchan->sapi_cmds);
 		mph_info_chan_confirm(lchan, PRIM_INFO_DEACTIVATE, 0);
@@ -1775,8 +1750,7 @@ static int lchan_deactivate_sapis(struct gsm_lchan *lchan)
 
 	/* nothing was queued */
 	if (res == 0) {
-		LOGP(DL1C, LOGL_ERROR, "%s all SAPIs already released?\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "all SAPIs already released?\n");
 		lchan_set_state(lchan, LCHAN_S_BROKEN);
 		mph_info_chan_confirm(lchan, PRIM_INFO_DEACTIVATE, 0);
 	}
@@ -1827,36 +1801,22 @@ int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type,
 }
 
 /* callback from OML */
-int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
-			struct tlv_parsed *new_attr, int kind, void *obj)
+int bts_model_apply_oml(struct gsm_bts *bts, const struct msgb *msg,
+			struct gsm_abis_mo *mo, void *obj)
 {
 	struct abis_om_fom_hdr *foh = msgb_l3(msg);
-	struct gsm_abis_mo *mo = gsm_objclass2mo(bts, foh->obj_class, &foh->obj_inst);
-	struct nm_fsm_ev_setattr_data ev_data = {
-		.msg = msg,
-		.cause = 0,
-	};
-	int rc;
 	struct gsm_bts_trx *trx;
 	struct lc15l1_hdl *fl1h;
-	uint8_t cell_size;
-
-	/* TODO: NM Object without FSM: */
-	switch (foh->obj_class) {
-	case NM_OC_GPRS_NSE:
-	case NM_OC_GPRS_CELL:
-	case NM_OC_GPRS_NSVC:
-		return oml_fom_ack_nack(ev_data.msg, ev_data.cause);
-	}
 
 	switch (foh->msg_type) {
 	case NM_MT_SET_RADIO_ATTR:
 		trx = obj;
 		fl1h = trx_lc15l1_hdl(trx);
-		/* convert max TA to max cell size in qbits */
-		cell_size = bts->max_ta << 2;
 
 #if LITECELL15_API_VERSION >= LITECELL15_API(2,1,7)
+	{
+		/* convert max TA to max cell size in qbits */
+		uint8_t cell_size = bts->max_ta << 2;
 		/* We do not need to check for L1 handle
 		 * because the max cell size parameter can receive before MphInit */
 		if (fl1h->phy_inst->u.lc15.max_cell_size != cell_size) {
@@ -1865,6 +1825,7 @@ int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
 			/* update current max cell size */
 			fl1h->phy_inst->u.lc15.max_cell_size = cell_size;
 		}
+	}
 #endif
 
 		/* Did we go through MphInit yet? If yes fire and forget */
@@ -1889,11 +1850,7 @@ int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
 		break;
 	}
 
-	rc = osmo_fsm_inst_dispatch(mo->fi,
-				    ev_data.cause == 0 ? NM_EV_SETATTR_ACK : NM_EV_SETATTR_NACK,
-				    &ev_data);
-	/* msgb ownership is transferred to FSM if it received ev: */
-	return rc == 0 ? 1 : 0;
+	return 0;
 }
 
 /* callback from OML */
@@ -1907,35 +1864,21 @@ int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo,
 
 	switch (mo->obj_class) {
 	case NM_OC_SITE_MANAGER:
-		rc = osmo_fsm_inst_dispatch(bts->site_mgr.mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
 	case NM_OC_BTS:
-		rc = osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_OPSTART_ACK, NULL);
+	case NM_OC_BASEB_TRANSC:
+	case NM_OC_GPRS_NSE:
+	case NM_OC_GPRS_CELL:
+	case NM_OC_GPRS_NSVC:
+		rc = osmo_fsm_inst_dispatch(mo->fi, NM_EV_OPSTART_ACK, NULL);
 		break;
 	case NM_OC_RADIO_CARRIER:
 		trx = (struct gsm_bts_trx *) obj;
 		rc = trx_init(trx);
 		break;
-	case NM_OC_BASEB_TRANSC:
-		bb_transc = (struct gsm_bts_bb_trx *) obj;
-		rc = osmo_fsm_inst_dispatch(bb_transc->mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
 	case NM_OC_CHANNEL:
 		ts = (struct gsm_bts_trx_ts*) obj;
 		rc = ts_opstart(ts);
 		break;
-	case NM_OC_GPRS_NSE:
-	case NM_OC_GPRS_CELL:
-	case NM_OC_GPRS_NSVC:
-		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, -1, -1);
-		rc = oml_mo_opstart_ack(mo);
-		if (mo->obj_class == NM_OC_BTS) {
-			oml_mo_state_chg(&bts->mo, -1, NM_AVSTATE_OK, -1);
-			oml_mo_state_chg(&bts->gprs.nse.mo, -1, NM_AVSTATE_OK, -1);
-			oml_mo_state_chg(&bts->gprs.cell.mo, -1, NM_AVSTATE_OK, -1);
-			oml_mo_state_chg(&bts->gprs.nsvc[0].mo, -1, NM_AVSTATE_OK, -1);
-		}
-		break;
 	default:
 		rc = oml_mo_opstart_nack(mo, NM_NACK_OBJCLASS_NOTSUPP);
 	}
@@ -2005,14 +1948,10 @@ int l1if_rsl_chan_act(struct gsm_lchan *lchan)
  */
 int l1if_rsl_chan_mod(struct gsm_lchan *lchan)
 {
-	const struct lchan_sapis *s4l = &sapis_for_lchan[lchan->type];
-	unsigned int i;
-
 	if (lchan->ho.active == HANDOVER_NONE)
 		return -1;
 
-	LOGP(DHO, LOGL_ERROR, "%s modifying channel for handover\n",
-		gsm_lchan_name(lchan));
+	LOGPLCHAN(lchan, DHO, LOGL_ERROR, "modifying channel for handover\n");
 
 	/* Give up listening to RACH bursts */
 	release_sapi_ul_rach(lchan);
@@ -2028,8 +1967,7 @@ int l1if_rsl_chan_rel(struct gsm_lchan *lchan)
 {
 	/* A duplicate RF Release Request, ignore it */
 	if (lchan->state == LCHAN_S_REL_REQ) {
-		LOGP(DL1C, LOGL_ERROR, "%s already in release request state.\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "already in release request state.\n");
 		return 0;
 	}
 
@@ -2065,8 +2003,7 @@ static int ts_disconnect_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 	struct gsm_bts_trx_ts *ts = &trx->ts[cnf->u8Tn];
 	OSMO_ASSERT(cnf->u8Tn < TRX_NR_TS);
 
-	LOGP(DL1C, LOGL_DEBUG, "%s Rx mphDisconnectCnf\n",
-	     gsm_lchan_name(ts->lchan));
+	LOGPLCHAN(ts->lchan, DL1C, LOGL_DEBUG, "Rx mphDisconnectCnf\n");
 
 	cb_ts_disconnected(ts);
 
@@ -2081,7 +2018,7 @@ int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts)
 	struct lc15l1_hdl *fl1h = trx_lc15l1_hdl(ts->trx);
 	GsmL1_MphDisconnectReq_t *cr;
 
-	DEBUGP(DRSL, "%s TS disconnect\n", gsm_lchan_name(ts->lchan));
+	LOGPLCHAN(ts->lchan, DRSL, LOGL_DEBUG, "TS disconnect\n");
 	cr = prim_init(msgb_l1prim(msg), GsmL1_PrimId_MphDisconnectReq, fl1h,
 		       l1p_handle_for_ts(ts));
 	cr->u8Tn = ts->nr;
@@ -2097,8 +2034,7 @@ static int ts_connect_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 	struct gsm_bts_trx_ts *ts = &trx->ts[cnf->u8Tn];
 	OSMO_ASSERT(cnf->u8Tn < TRX_NR_TS);
 
-	DEBUGP(DL1C, "%s %s Rx mphConnectCnf flags=%s%s%s\n",
-	       gsm_lchan_name(ts->lchan),
+	LOGPLCHAN(ts->lchan, DL1C, LOGL_DEBUG, "%s Rx mphConnectCnf flags=%s%s%s\n",
 	       gsm_pchan_name(ts->pchan),
 	       ts->flags & TS_F_PDCH_ACTIVE ? "ACTIVE " : "",
 	       ts->flags & TS_F_PDCH_ACT_PENDING ? "ACT_PENDING " : "",
