@@ -589,11 +589,10 @@ static int ph_tch_req(struct gsm_bts_trx *trx, struct msgb *msg,
 	if (msg) {
 		nmsg = l1p_msgb_alloc();
 		if (!nmsg) {
-			LOGPFN(DL1C, LOGL_FATAL, u32Fn, "L1SAP PH-TCH.req msg alloc failed\n");
+			LOGPLCFN(lchan, u32Fn, DL1C, LOGL_FATAL, "L1SAP PH-TCH.req msg alloc failed\n");
 			return -ENOMEM;
 		}
 
-		msgb_pull(msg, sizeof(*l1sap));
 		tOCTVC1_GSM_MSG_TRX_REQUEST_LOGICAL_CHANNEL_DATA_CMD *data_req =
 			(tOCTVC1_GSM_MSG_TRX_REQUEST_LOGICAL_CHANNEL_DATA_CMD *)
 			msgb_put(nmsg, sizeof(*data_req));
@@ -615,7 +614,7 @@ static int ph_tch_req(struct gsm_bts_trx *trx, struct msgb *msg,
 				&data_req->Data.ulPayloadType,
 				data_req->Data.abyDataContent,
 				&data_req->Data.ulDataLength,
-				msg->data, msg->len);
+				msgb_l2(msg), msgb_l2len(msg));
 
 		mOCTVC1_GSM_MSG_TRX_REQUEST_LOGICAL_CHANNEL_DATA_CMD_SWAP(data_req);
 	} else {
@@ -773,11 +772,12 @@ int bts_model_init(struct gsm_bts *bts)
 	/* FIXME: what is the nominal transmit power of the PHY/board? */
 	bts->c0->nominal_power = 15;
 
-	osmo_bts_set_feature(bts->features, BTS_FEAT_GPRS);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_OML_ALERTS);
+	/* order alphabetically */
 #if defined(cOCTVC1_GSM_LOGICAL_CHANNEL_COMBINATION_ENUM_FCCH_SCH_BCCH_CCCH_SDCCH4_CBCH_SACCHC4) && defined(cOCTVC1_GSM_LOGICAL_CHANNEL_COMBINATION_ENUM_SDCCH8_CBCH_SACCHC8)
 	osmo_bts_set_feature(bts->features, BTS_FEAT_CBCH);
 #endif
+	osmo_bts_set_feature(bts->features, BTS_FEAT_GPRS);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_OML_ALERTS);
 	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_V1);
 	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_H_V1);
 
@@ -1302,7 +1302,11 @@ static int retransmit_wlc_upto(struct octphy_hdl *fl1h, uint32_t trans_id)
 			wlc->num_retrans++;
 			msg = msgb_copy(wlc->cmd_msg, "PHY CMD Retrans");
 			msg_set_retrans_flag(msg);
-			osmo_wqueue_enqueue(&fl1h->phy_wq, msg);
+			if (osmo_wqueue_enqueue(&fl1h->phy_wq, msg) < 0) {
+				LOGP(DL1C, LOGL_ERROR, "Queue full on wlc retransmit\n");
+				msgb_free(msg);
+				return 0;
+			}
 			osmo_timer_schedule(&wlc->timer, CMD_TIMEOUT, 0);
 			count++;
 			LOGP(DL1C, LOGL_INFO, "Re-transmitting %s "
