@@ -13,7 +13,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -344,7 +344,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		abort();
 	}
 
-	len = (msg->l2h) ? msgb_l2len(msg) : 0;
+	len = msgb_l2len(msg);
 
 	chan_nr = l1sap->u.data.chan_nr;
 	link_id = l1sap->u.data.link_id;
@@ -604,15 +604,6 @@ static int mph_info_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		else
 			l1if_rsl_chan_rel(lchan);
 		break;
-	case PRIM_INFO_ACT_UL_ACC:
-	case PRIM_INFO_DEACT_UL_ACC:
-		chan_nr = l1sap->u.info.u.ulacc_req.chan_nr;
-		lchan = get_lchan_by_chan_nr(trx, chan_nr);
-		if (l1sap->u.info.type == PRIM_INFO_ACT_UL_ACC)
-			l1if_set_ul_acc(lchan, true);
-		else
-			l1if_set_ul_acc(lchan, false);
-		break;
 	default:
 		LOGP(DL1C, LOGL_NOTICE, "unknown MPH-INFO.req %d\n",
 			l1sap->u.info.type);
@@ -1947,27 +1938,5 @@ int bts_model_phy_link_open(struct phy_link *plink)
 	hdl = pinst->u.sysmobts.hdl;
 	osmo_strlcpy(bts->sub_model, sysmobts_model(hdl->hw_info.model_nr, hdl->hw_info.trx_nr), sizeof(bts->sub_model));
 
-	/* Frequency bands indicated to the BSC */
-	for (unsigned int i = 0; i < sizeof(hdl->hw_info.band_support) * 8; i++) {
-		if (~hdl->hw_info.band_support & (1 << i))
-			continue;
-		switch (1 << i) {
-		case GSM_BAND_850:
-			pinst->trx->support.freq_bands |= NM_IPAC_F_FREQ_BAND_850;
-			break;
-		case GSM_BAND_900:
-			pinst->trx->support.freq_bands |= NM_IPAC_F_FREQ_BAND_PGSM;
-			/* XXX: does GSM_BAND_900 include NM_IPAC_F_FREQ_BAND_EGSM? */
-			/* XXX: does GSM_BAND_900 include NM_IPAC_F_FREQ_BAND_RGSM? */
-			break;
-		case GSM_BAND_1800:
-			pinst->trx->support.freq_bands |= NM_IPAC_F_FREQ_BAND_DCS;
-			break;
-		case GSM_BAND_1900:
-			pinst->trx->support.freq_bands |= NM_IPAC_F_FREQ_BAND_PCS;
-			break;
-		}
-	}
-
 	return 0;
 }
