@@ -45,6 +45,7 @@
 #include <osmo-bts/scheduler.h>
 #include <osmo-bts/pcu_if.h>
 #include <osmo-bts/nm_common_fsm.h>
+#include <osmo-bts/handover.h>
 
 #include "l1_if.h"
 #include "trx_if.h"
@@ -89,16 +90,21 @@ struct trx_l1h *trx_l1h_alloc(void *tall_ctx, struct phy_instance *pinst)
 
 int bts_model_lchan_deactivate(struct gsm_lchan *lchan)
 {
-	if (lchan->rel_act_kind == LCHAN_REL_ACT_REACT) {
-		lchan->rel_act_kind = LCHAN_REL_ACT_RSL;
-		/* FIXME: perform whatever is needed (if any) to set proper PCH/AGCH allocation according to
-		   3GPP TS 44.018 Table 10.5.2.11.1 using num_agch(lchan->ts->trx, "TRX L1"); function */
-		return 0;
-	}
+	int rc;
 	/* set lchan inactive */
 	lchan_set_state(lchan, LCHAN_S_NONE);
 
-	return trx_sched_set_lchan(lchan, gsm_lchan2chan_nr(lchan), LID_DEDIC, false);
+	/* Disable it on the scheduler: */
+	rc = trx_sched_set_lchan(lchan, gsm_lchan2chan_nr(lchan), LID_DEDIC, false);
+
+	/* Reactivate CCCH due to SI3 update in RSL */
+	if (lchan->rel_act_kind == LCHAN_REL_ACT_REACT) {
+		lchan->rel_act_kind = LCHAN_REL_ACT_RSL;
+		trx_sched_set_lchan(lchan, gsm_lchan2chan_nr(lchan), LID_DEDIC, true);
+		lchan_set_state(lchan, LCHAN_S_ACTIVE);
+		return rc;
+	}
+	return rc;
 }
 
 int bts_model_lchan_deactivate_sacch(struct gsm_lchan *lchan)
@@ -195,7 +201,7 @@ void bts_model_abis_close(struct gsm_bts *bts)
 {
 	/* Go into shutdown state deactivating transceivers until Abis link
 	 * becomes up again */
-	bts_shutdown_ext(bts, "Abis close", false);
+	bts_shutdown_ext(bts, "Abis close", false, true);
 }
 
 int bts_model_adjst_ms_pwr(struct gsm_lchan *lchan)
@@ -206,7 +212,7 @@ int bts_model_adjst_ms_pwr(struct gsm_lchan *lchan)
 }
 
 /* set bts attributes */
-static uint8_t trx_set_bts(struct gsm_bts *bts, struct tlv_parsed *new_attr)
+static uint8_t trx_set_bts(struct gsm_bts *bts)
 {
 	struct phy_instance *pinst = trx_phy_instance(bts->c0);
 	struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
@@ -270,11 +276,26 @@ static uint8_t trx_set_ts_as_pchan(struct gsm_bts_trx_ts *ts,
 	if (rc)
 		return NM_NACK_RES_NOTAVAIL;
 
-	/* activate lchan for CCCH */
-	if (pchan == GSM_PCHAN_CCCH || pchan == GSM_PCHAN_CCCH_SDCCH4 ||
-	    pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH) {
+	/* activate lchans for [CBCH/]BCCH/CCCH */
+	switch (pchan) {
+	case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
+		/* using RSL_CHAN_OSMO_CBCH4 is correct here, because the scheduler
+		 * does not distinguish between SDCCH/4+CBCH abd SDCCH/8+CBCH. */
+		trx_sched_set_lchan(&ts->lchan[CBCH_LCHAN],
+				    RSL_CHAN_OSMO_CBCH4, LID_DEDIC, true);
+		break;
+	case GSM_PCHAN_CCCH_SDCCH4_CBCH:
+		trx_sched_set_lchan(&ts->lchan[CBCH_LCHAN],
+				    RSL_CHAN_OSMO_CBCH4, LID_DEDIC, true);
+		/* fall-through */
+	case GSM_PCHAN_CCCH_SDCCH4:
+	case GSM_PCHAN_CCCH:
+		trx_sched_set_bcch_ccch(&ts->lchan[CCCH_LCHAN], true);
 		ts->lchan[CCCH_LCHAN].rel_act_kind = LCHAN_REL_ACT_OML;
 		lchan_set_state(&ts->lchan[CCCH_LCHAN], LCHAN_S_ACTIVE);
+		break;
+	default:
+		break;
 	}
 
 	slottype = transceiver_chan_types[pchan];
@@ -326,8 +347,8 @@ static uint8_t trx_set_ts(struct gsm_bts_trx_ts *ts)
 /* enable ciphering */
 static int l1if_set_ciphering(struct gsm_lchan *lchan, uint8_t chan_nr, int downlink)
 {
-	/* ciphering already enabled in both directions */
-	if (lchan->ciph_state == LCHAN_CIPH_RXTX_CONF)
+	/* ignore the request when the channel is not active */
+	if (lchan->state != LCHAN_S_ACTIVE)
 		return -EINVAL;
 
 	if (!downlink) {
@@ -396,86 +417,88 @@ int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 		/* put data into scheduler's queue */
 		return trx_sched_tch_req(trx, l1sap);
 	case OSMO_PRIM(PRIM_MPH_INFO, PRIM_OP_REQUEST):
+		if (l1sap->u.info.type == PRIM_INFO_ACT_CIPH)
+			chan_nr = l1sap->u.info.u.ciph_req.chan_nr;
+		else /* u.act_req used by PRIM_INFO_{ACTIVATE,DEACTIVATE,MODIFY} */
+			chan_nr = l1sap->u.info.u.act_req.chan_nr;
+		lchan = get_lchan_by_chan_nr(trx, chan_nr);
+		if (OSMO_UNLIKELY(lchan == NULL)) {
+			LOGP(DL1C, LOGL_ERROR,
+			     "Rx MPH-INFO.req (type=0x%02x) for non-existent lchan (%s)\n",
+			     l1sap->u.info.type, rsl_chan_nr_str(chan_nr));
+			rc = -ENODEV;
+			break;
+		}
+
 		switch (l1sap->u.info.type) {
 		case PRIM_INFO_ACT_CIPH:
-			chan_nr = l1sap->u.info.u.ciph_req.chan_nr;
-			lchan = get_lchan_by_chan_nr(trx, chan_nr);
 			if (l1sap->u.info.u.ciph_req.uplink)
 				l1if_set_ciphering(lchan, chan_nr, 0);
 			if (l1sap->u.info.u.ciph_req.downlink)
 				l1if_set_ciphering(lchan, chan_nr, 1);
 			break;
 		case PRIM_INFO_ACTIVATE:
-		case PRIM_INFO_DEACTIVATE:
+			if ((chan_nr & 0xE0) == 0x80) {
+				LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Cannot activate"
+					  " channel %s\n", rsl_chan_nr_str(chan_nr));
+				rc = -EPERM;
+				break;
+			}
+
+			/* activate dedicated channel */
+			trx_sched_set_lchan(lchan, chan_nr, LID_DEDIC, true);
+			/* activate associated channel */
+			trx_sched_set_lchan(lchan, chan_nr, LID_SACCH, true);
+			/* set mode */
+			trx_sched_set_mode(lchan->ts, chan_nr,
+					   lchan->rsl_cmode, lchan->tch_mode,
+					   lchan->tch.amr_mr.num_modes,
+					   lchan->tch.amr_mr.mode[0].mode,
+					   lchan->tch.amr_mr.mode[1].mode,
+					   lchan->tch.amr_mr.mode[2].mode,
+					   lchan->tch.amr_mr.mode[3].mode,
+					   amr_get_initial_mode(lchan),
+					   (lchan->ho.active == HANDOVER_ENABLED) ||
+					    rsl_chan_rt_is_asci(lchan->rsl_chan_rt));
+			/* set lchan active */
+			lchan_set_state(lchan, LCHAN_S_ACTIVE);
+			/* set initial ciphering */
+			l1if_set_ciphering(lchan, chan_nr, 0);
+			l1if_set_ciphering(lchan, chan_nr, 1);
+			if (lchan->encr.alg_id)
+				lchan->ciph_state = LCHAN_CIPH_RXTX_CONF;
+			else
+				lchan->ciph_state = LCHAN_CIPH_NONE;
+
+			/* confirm */
+			mph_info_chan_confirm(trx, chan_nr, PRIM_INFO_ACTIVATE, 0);
+			break;
 		case PRIM_INFO_MODIFY:
-			chan_nr = l1sap->u.info.u.act_req.chan_nr;
-			lchan = get_lchan_by_chan_nr(trx, chan_nr);
-			if (l1sap->u.info.type == PRIM_INFO_ACTIVATE) {
-				if ((chan_nr & 0xE0) == 0x80) {
-					LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Cannot activate"
-						  " channel %s\n", rsl_chan_nr_str(chan_nr));
-					break;
-				}
-
-				/* attempt to allocate an Error Concealment Unit instance, if available */
-				lchan->ecu_state = osmo_ecu_init(trx, lchan2ecu_codec(lchan));
-
-				/* activate dedicated channel */
-				trx_sched_set_lchan(lchan, chan_nr, LID_DEDIC, true);
-				/* activate associated channel */
-				trx_sched_set_lchan(lchan, chan_nr, LID_SACCH, true);
-				/* set mode */
-				trx_sched_set_mode(lchan->ts, chan_nr,
-					lchan->rsl_cmode, lchan->tch_mode,
-					lchan->tch.amr_mr.num_modes,
-					lchan->tch.amr_mr.mode[0].mode,
-					lchan->tch.amr_mr.mode[1].mode,
-					lchan->tch.amr_mr.mode[2].mode,
-					lchan->tch.amr_mr.mode[3].mode,
-					amr_get_initial_mode(lchan),
-					(lchan->ho.active == 1));
-				/* set lchan active */
-				lchan_set_state(lchan, LCHAN_S_ACTIVE);
-				/* set initial ciphering */
-				l1if_set_ciphering(lchan, chan_nr, 0);
-				l1if_set_ciphering(lchan, chan_nr, 1);
-				if (lchan->encr.alg_id)
-					lchan->ciph_state = LCHAN_CIPH_RXTX_CONF;
-				else
-					lchan->ciph_state = LCHAN_CIPH_NONE;
-
-				/* confirm */
-				mph_info_chan_confirm(trx, chan_nr, PRIM_INFO_ACTIVATE, 0);
-				break;
-			}
-			if (l1sap->u.info.type == PRIM_INFO_MODIFY) {
-				/* ECU for possibly new codec */
-				if (lchan->ecu_state)
-					osmo_ecu_destroy(lchan->ecu_state);
-				lchan->ecu_state = osmo_ecu_init(trx, lchan2ecu_codec(lchan));
-				/* change mode */
-				trx_sched_set_mode(lchan->ts, chan_nr,
-					lchan->rsl_cmode, lchan->tch_mode,
-					lchan->tch.amr_mr.num_modes,
-					lchan->tch.amr_mr.mode[0].mode,
-					lchan->tch.amr_mr.mode[1].mode,
-					lchan->tch.amr_mr.mode[2].mode,
-					lchan->tch.amr_mr.mode[3].mode,
-					amr_get_initial_mode(lchan),
-					0);
-				break;
-			}
-			/* here, type == PRIM_INFO_DEACTIVATE */
+			/* change mode */
+			trx_sched_set_mode(lchan->ts, chan_nr,
+					   lchan->rsl_cmode, lchan->tch_mode,
+					   lchan->tch.amr_mr.num_modes,
+					   lchan->tch.amr_mr.mode[0].mode,
+					   lchan->tch.amr_mr.mode[1].mode,
+					   lchan->tch.amr_mr.mode[2].mode,
+					   lchan->tch.amr_mr.mode[3].mode,
+					   amr_get_initial_mode(lchan),
+					   0);
+			/* update ciphering params */
+			l1if_set_ciphering(lchan, chan_nr, 0);
+			l1if_set_ciphering(lchan, chan_nr, 1);
+			if (lchan->encr.alg_id)
+				lchan->ciph_state = LCHAN_CIPH_RXTX_CONF;
+			else
+				lchan->ciph_state = LCHAN_CIPH_NONE;
+			break;
+		case PRIM_INFO_DEACTIVATE:
 			if ((chan_nr & 0xE0) == 0x80) {
 				LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Cannot deactivate"
 					  " channel %s\n", rsl_chan_nr_str(chan_nr));
+				rc = -EPERM;
 				break;
 			}
-			/* clear ECU state (if any) */
-			if (lchan->ecu_state) {
-				osmo_ecu_destroy(lchan->ecu_state);
-				lchan->ecu_state = NULL;
-			}
 			/* deactivate associated channel */
 			bts_model_lchan_deactivate_sacch(lchan);
 			if (!l1sap->u.info.u.act_req.sacch_only) {
@@ -520,79 +543,52 @@ int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type,
 }
 
 /* callback from OML */
-int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
-			struct tlv_parsed *new_attr, int kind, void *obj)
+int bts_model_apply_oml(struct gsm_bts *bts, const struct msgb *msg,
+			struct gsm_abis_mo *mo, void *obj)
 {
 	struct abis_om_fom_hdr *foh = msgb_l3(msg);
-	struct gsm_abis_mo *mo = gsm_objclass2mo(bts, foh->obj_class, &foh->obj_inst);
-	struct nm_fsm_ev_setattr_data ev_data = {
-		.msg = msg,
-		.cause = 0,
-	};
 	int rc;
 
-	/* TODO: NM Object without FSM: */
-	switch (foh->obj_class) {
-	case NM_OC_GPRS_NSE:
-	case NM_OC_GPRS_CELL:
-	case NM_OC_GPRS_NSVC:
-		return oml_fom_ack_nack(ev_data.msg, ev_data.cause);
-	}
-
 	switch (foh->msg_type) {
 	case NM_MT_SET_BTS_ATTR:
-		ev_data.cause = trx_set_bts(obj, new_attr);
+		rc = trx_set_bts(obj);
 		break;
 	case NM_MT_SET_RADIO_ATTR:
-		ev_data.cause = trx_set_trx(obj);
+		rc = trx_set_trx(obj);
 		break;
 	case NM_MT_SET_CHAN_ATTR:
-		ev_data.cause = trx_set_ts(obj);
+		rc = trx_set_ts(obj);
+		break;
+	default:
+		rc = 0;
 		break;
 	}
 
-	rc = osmo_fsm_inst_dispatch(mo->fi,
-				    ev_data.cause == 0 ? NM_EV_SETATTR_ACK : NM_EV_SETATTR_NACK,
-				    &ev_data);
-	/* msgb ownsership is transferred to FSM if it received ev: */
-	return rc == 0 ? 1 : 0;
+	return rc;
 }
 
 /* callback from OML */
 int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo,
 		      void *obj)
 {
-	struct gsm_bts_bb_trx *bb_transc;
 	struct gsm_bts_trx *trx;
-	struct gsm_bts_trx_ts *ts;
 	int rc;
 
 	switch (mo->obj_class) {
 	case NM_OC_SITE_MANAGER:
-		rc = osmo_fsm_inst_dispatch(bts->site_mgr.mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
 	case NM_OC_BTS:
-		rc = osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_OPSTART_ACK, NULL);
+	case NM_OC_BASEB_TRANSC:
+	case NM_OC_CHANNEL:
+	case NM_OC_GPRS_NSE:
+	case NM_OC_GPRS_CELL:
+	case NM_OC_GPRS_NSVC:
+		rc = osmo_fsm_inst_dispatch(mo->fi, NM_EV_OPSTART_ACK, NULL);
 		break;
 	case NM_OC_RADIO_CARRIER:
 		/* activate transceiver */
 		trx = (struct gsm_bts_trx *) obj;
 		rc = trx_init(trx);
 		break;
-	case NM_OC_BASEB_TRANSC:
-		bb_transc = (struct gsm_bts_bb_trx *) obj;
-		rc = osmo_fsm_inst_dispatch(bb_transc->mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
-	case NM_OC_CHANNEL:
-		ts = (struct gsm_bts_trx_ts *) obj;
-		rc = osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
-	case NM_OC_GPRS_NSE:
-	case NM_OC_GPRS_CELL:
-	case NM_OC_GPRS_NSVC:
-		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, NM_AVSTATE_OK, -1);
-		rc = oml_mo_opstart_ack(mo);
-		break;
 	default:
 		rc = oml_mo_opstart_nack(mo, NM_NACK_OBJCLASS_NOTSUPP);
 	}
