@@ -70,6 +70,10 @@ static const struct rate_ctr_desc btstrx_ctr_desc[] = {
 		"trx_sched:dl_fh_no_carrier",
 		"Frequency hopping: no carrier found for a Downlink burst (check hopping parameters)"
 	},
+	[BTSTRX_CTR_SCHED_DL_FH_CACHE_MISS] = {
+		"trx_sched:dl_fh_cache_miss",
+		"Frequency hopping: no Downlink carrier found in cache (lookup performed)"
+	},
 	[BTSTRX_CTR_SCHED_UL_FH_NO_CARRIER] = {
 		"trx_sched:ul_fh_no_carrier",
 		"Frequency hopping: no carrier found for an Uplink burst (check hopping parameters)"
@@ -134,25 +138,36 @@ int bts_model_init(struct gsm_bts *bts)
 	 */
 	bts->c0->nominal_power = 23;
 
-	osmo_bts_set_feature(bts->features, BTS_FEAT_GPRS);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_EGPRS);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_OML_ALERTS);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_V1);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_H_V1);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_EFR);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_AMR);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_H_AMR);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_CBCH);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_HOPPING);
+	/* order alphabetically */
 	osmo_bts_set_feature(bts->features, BTS_FEAT_ACCH_REP);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_MULTI_TSC);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_VAMOS);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_BCCH_POWER_RED);
 	osmo_bts_set_feature(bts->features, BTS_FEAT_ACCH_TEMP_OVP);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_BCCH_POWER_RED);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_CBCH);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_EGPRS);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_GPRS);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_HOPPING);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_MULTI_TSC);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_OML_ALERTS);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_AMR);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_EFR);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_V1);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_H_AMR);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_H_V1);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_VAMOS);
 
 	bts_internal_flag_set(bts, BTS_INTERNAL_FLAG_MEAS_PAYLOAD_COMB);
 	bts_internal_flag_set(bts, BTS_INTERNAL_FLAG_INTERF_MEAS);
 
+	/* The default HR codec output format in the absence of saved
+	 * vty config needs to match what was implemented previously,
+	 * for the sake of existing deployments, i.e., to avoid
+	 * a surprise functional change upon software update. */
+	bts->emit_hr_rfc5993 = true;
+
+	/* For the same reason as the above, rtp internal-uplink-ecu
+	 * needs to be enabled by default on osmo-bts-trx model only. */
+	bts->use_ul_ecu = true;
+
 	return 0;
 }
 
