@@ -1,7 +1,7 @@
 /*
  * (C) 2013 by Andreas Eversberg <jolly@eversberg.eu>
  * (C) 2015-2017 by Harald Welte <laforge@gnumonks.org>
- * Contributions by sysmocom - s.f.m.c. GmbH
+ * (C) 2020-2023 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  *
  * All Rights Reserved
  *
@@ -30,7 +30,6 @@
 #include <osmocom/gsm/gsm0502.h>
 
 #include <osmocom/codec/codec.h>
-#include <osmocom/codec/ecu.h>
 
 #include <osmocom/coding/gsm0503_coding.h>
 #include <osmocom/coding/gsm0503_amr_dtx.h>
@@ -70,22 +69,50 @@ static const uint8_t sched_tchf_dl_amr_cmi_map[26] = {
 
 extern const uint8_t sched_tchh_dl_amr_cmi_map[26];
 
-/*! \brief a single TCH/F burst was received by the PHY, process it */
+static int decode_fr_facch(struct l1sched_ts *l1ts,
+			   const struct trx_ul_burst_ind *bi)
+{
+	struct l1sched_chan_state *chan_state = &l1ts->chan_state[bi->chan];
+	const sbit_t *bursts_p = chan_state->ul_bursts;
+	struct l1sched_meas_set meas_avg;
+	uint8_t data[GSM_MACBLOCK_LEN];
+	int n_errors, n_bits_total;
+	int rc;
+
+	rc = gsm0503_tch_fr_facch_decode(&data[0], BUFTAIL8(bursts_p),
+					 &n_errors, &n_bits_total);
+	if (rc != GSM_MACBLOCK_LEN)
+		return rc;
+
+	/* average measurements of the last 8 bursts, obtain TDMA Fn of the first burst */
+	trx_sched_meas_avg(chan_state, &meas_avg, SCHED_MEAS_AVG_M_S8N8);
+
+	_sched_compose_ph_data_ind(l1ts, meas_avg.fn, bi->chan,
+				   &data[0], GSM_MACBLOCK_LEN,
+				   compute_ber10k(n_bits_total, n_errors),
+				   meas_avg.rssi,
+				   meas_avg.toa256,
+				   meas_avg.ci_cb,
+				   PRES_INFO_UNKNOWN);
+	return GSM_MACBLOCK_LEN;
+}
+
+/* Process a single Uplink TCH/F burst received by the PHY.
+ * This function is visualized in file 'doc/trx_sched_tch.txt'. */
 int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 {
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[bi->chan];
 	struct gsm_lchan *lchan = chan_state->lchan;
-	sbit_t *burst, **bursts_p = &chan_state->ul_bursts;
-	uint8_t *mask = &chan_state->ul_mask;
+	sbit_t *burst, *bursts_p = chan_state->ul_bursts;
+	uint32_t *mask = &chan_state->ul_mask;
 	uint8_t rsl_cmode = chan_state->rsl_cmode;
 	uint8_t tch_mode = chan_state->tch_mode;
-	uint8_t tch_data[128]; /* just to be safe */
+	uint8_t tch_data[290]; /* large enough to hold 290 unpacked bits for CSD */
 	enum sched_meas_avg_mode meas_avg_mode = SCHED_MEAS_AVG_M_S8N8;
 	struct l1sched_meas_set meas_avg;
 	int rc, amr = 0;
 	int n_errors = 0;
 	int n_bits_total = 0;
-	bool bfi_flag = false;
 	unsigned int fn_begin;
 	uint16_t ber10k;
 	uint8_t is_sub = 0;
@@ -99,17 +126,10 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi, "Received TCH/F, bid=%u\n", bi->bid);
 
-	/* allocate burst memory, if not already */
-	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(l1ts, 928);
-		if (!*bursts_p)
-			return -ENOMEM;
-	}
-
 	/* shift the buffer by 4 bursts leftwards */
 	if (bi->bid == 0) {
-		memcpy(*bursts_p, *bursts_p + 464, 464);
-		memset(*bursts_p + 464, 0, 464);
+		memmove(BUFPOS(bursts_p, 0), BUFPOS(bursts_p, 4), 20 * BPLEN);
+		memset(BUFPOS(bursts_p, 20), 0, 4 * BPLEN);
 		*mask = *mask << 4;
 	}
 
@@ -119,13 +139,12 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	/* store measurements */
 	trx_sched_meas_push(chan_state, bi);
 
-	/* copy burst to end of buffer of 8 bursts */
-	burst = *bursts_p + bi->bid * 116 + 464;
+	/* copy burst to end of buffer of 24 bursts */
+	burst = BUFPOS(bursts_p, 20 + bi->bid);
 	if (bi->burst_len > 0) {
 		memcpy(burst, bi->burst + 3, 58);
 		memcpy(burst + 58, bi->burst + 87, 58);
-	} else
-		memset(burst, 0, 116);
+	}
 
 	/* wait until complete set of bursts */
 	if (bi->bid != 3)
@@ -139,17 +158,22 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		return 0; /* TODO: send BFI */
 	}
 
-	/* decode
-	 * also shift buffer by 4 bursts for interleaving */
+	/* TCH/F: speech and signalling frames are interleaved over 8 bursts, while
+	 * CSD frames are interleaved over 22 bursts.  Unless we're in CSD mode,
+	 * decode only the last 8 bursts to avoid introducing additional delays. */
 	switch (tch_mode) {
 	case GSM48_CMODE_SIGN:
 	case GSM48_CMODE_SPEECH_V1: /* FR */
-		rc = gsm0503_tch_fr_decode(tch_data, *bursts_p, 1, 0, &n_errors, &n_bits_total);
+		rc = gsm0503_tch_fr_decode(tch_data, BUFTAIL8(bursts_p),
+					   1, 0, &n_errors, &n_bits_total);
 		if (rc == GSM_FR_BYTES) /* only for valid *speech* frames */
-			lchan_set_marker(osmo_fr_check_sid(tch_data, rc), lchan); /* DTXu */
+			lchan_set_marker(osmo_fr_is_any_sid(tch_data), lchan); /* DTXu */
 		break;
 	case GSM48_CMODE_SPEECH_EFR: /* EFR */
-		rc = gsm0503_tch_fr_decode(tch_data, *bursts_p, 1, 1, &n_errors, &n_bits_total);
+		rc = gsm0503_tch_fr_decode(tch_data, BUFTAIL8(bursts_p),
+					   1, 1, &n_errors, &n_bits_total);
+		if (rc == GSM_EFR_BYTES) /* only for valid *speech* frames */
+			lchan_set_marker(osmo_efr_is_any_sid(tch_data), lchan); /* DTXu */
 		break;
 	case GSM48_CMODE_SPEECH_AMR: /* AMR */
 		/* the first FN 0,8,17 defines that CMI is included in frame,
@@ -173,7 +197,7 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		 * we receive an FACCH frame instead of a voice frame (we
 		 * do not know this before we actually decode the frame) */
 		amr = sizeof(struct amr_hdr);
-		rc = gsm0503_tch_afs_decode_dtx(tch_data + amr, *bursts_p,
+		rc = gsm0503_tch_afs_decode_dtx(tch_data + amr, BUFTAIL8(bursts_p),
 			amr_is_cmr, chan_state->codec, chan_state->codecs, &chan_state->ul_ft,
 			&chan_state->ul_cmr, &n_errors, &n_bits_total, &chan_state->amr_last_dtx);
 
@@ -225,6 +249,40 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 			        ft, AMR_GOOD);
 		}
 
+		break;
+	/* CSD (TCH/F9.6): 12.0 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_12k0:
+		/* FACCH/F does not steal TCH/F9.6 frames, but only disturbs some bits */
+		decode_fr_facch(l1ts, bi);
+		rc = gsm0503_tch_fr96_decode(&tch_data[0], BUFPOS(bursts_p, 0),
+					     &n_errors, &n_bits_total);
+		meas_avg_mode = SCHED_MEAS_AVG_M_S24N22;
+		break;
+	/* CSD (TCH/F4.8): 6.0 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_6k0:
+		/* FACCH/F does not steal TCH/F4.8 frames, but only disturbs some bits */
+		decode_fr_facch(l1ts, bi);
+		rc = gsm0503_tch_fr48_decode(&tch_data[0], BUFPOS(bursts_p, 0),
+					     &n_errors, &n_bits_total);
+		meas_avg_mode = SCHED_MEAS_AVG_M_S24N22;
+		break;
+	/* CSD (TCH/F2.4): 3.6 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_3k6:
+		/* TCH/F2.4 employs the same interleaving as TCH/FS (8 bursts),
+		 * so FACCH/F *does* steal TCH/F2.4 frames completely. */
+		if (decode_fr_facch(l1ts, bi) == GSM_MACBLOCK_LEN)
+			return 0; /* TODO: emit BFI */
+		rc = gsm0503_tch_fr24_decode(&tch_data[0], BUFTAIL8(bursts_p),
+					     &n_errors, &n_bits_total);
+		meas_avg_mode = SCHED_MEAS_AVG_M_S8N8;
+		break;
+	/* CSD (TCH/F14.4): 14.5 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_14k5:
+		/* FACCH/F does not steal TCH/F14.4 frames, but only disturbs some bits */
+		decode_fr_facch(l1ts, bi);
+		rc = gsm0503_tch_fr144_decode(&tch_data[0], BUFPOS(bursts_p, 0),
+					      &n_errors, &n_bits_total);
+		meas_avg_mode = SCHED_MEAS_AVG_M_S24N22;
 		break;
 	default:
 		LOGL1SB(DL1P, LOGL_ERROR, l1ts, bi,
@@ -233,6 +291,8 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		return -EINVAL;
 	}
 
+	ber10k = compute_ber10k(n_bits_total, n_errors);
+
 	/* average measurements of the last N (depends on mode) bursts */
 	trx_sched_meas_avg(chan_state, &meas_avg, meas_avg_mode);
 	/* meas_avg.fn now contains TDMA frame number of the first burst */
@@ -242,31 +302,18 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		trx_loop_amr_input(chan_state, &meas_avg);
 
 	/* Check if the frame is bad */
-	if (rc < 0) {
-		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi, "Received bad data (%u/%u)\n",
-			bi->fn % l1ts->mf_period, l1ts->mf_period);
-		bfi_flag = true;
-	} else if (rc < 4) {
+	if (rc < 4) {
 		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi,
-			"Received bad data (%u/%u) with invalid codec mode %d\n",
-			bi->fn % l1ts->mf_period, l1ts->mf_period, rc);
-		bfi_flag = true;
-	}
-
-	if (rc != GSM_MACBLOCK_LEN && lchan->ecu_state)
-		osmo_ecu_frame_in(lchan->ecu_state, bfi_flag, tch_data, rc);
-
-	ber10k = compute_ber10k(n_bits_total, n_errors);
-	if (bfi_flag)
-		goto bfi;
-
-	/* FACCH */
-	if (rc == GSM_MACBLOCK_LEN) {
+			BAD_DATA_MSG_FMT "\n", BAD_DATA_MSG_ARGS);
+		rc = 0;		/* this is how we signal BFI to l1sap */
+	} else if (rc == GSM_MACBLOCK_LEN) { /* FACCH/F */
 		_sched_compose_ph_data_ind(l1ts, fn_begin, bi->chan,
-			tch_data + amr, GSM_MACBLOCK_LEN,
-			meas_avg.rssi, meas_avg.toa256,
-			meas_avg.ci_cb, ber10k,
-			PRES_INFO_UNKNOWN);
+					   &tch_data[amr], GSM_MACBLOCK_LEN,
+					   ber10k,
+					   meas_avg.rssi,
+					   meas_avg.toa256,
+					   meas_avg.ci_cb,
+					   PRES_INFO_UNKNOWN);
 
 		/* If we are in SPEECH mode we will generate a fake (BFI) TCH
 		 * indication as well. This indication is needed by the higher
@@ -276,70 +323,28 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		 * the fake (BFI) TCH indication we set meas_avg.rssi to zero.
 		 * Doing so tells l1sap.c to ignore the measurement result. */
 		meas_avg.rssi = 0;
-
-bfi:
-		if (rsl_cmode == RSL_CMOD_SPD_SPEECH) {
-			/* indicate bad frame */
-			if (lchan->tch.dtx.ul_sid) {
-				/* DTXu: pause in progress. Push empty payload to upper layers */
-				rc = 0;
-				goto compose_l1sap;
-			}
-
-			/* If there is an ECU active on this channel, use its output */
-			if (lchan->ecu_state) {
-				rc = osmo_ecu_frame_out(lchan->ecu_state, tch_data);
-				if (rc >= 0) /* Otherwise we send a BFI */
-					goto compose_l1sap;
-			}
-
-			switch (tch_mode) {
-			case GSM48_CMODE_SPEECH_V1: /* FR */
-				memset(tch_data, 0, GSM_FR_BYTES);
-				tch_data[0] = 0xd0;
-				rc = GSM_FR_BYTES;
-				break;
-			case GSM48_CMODE_SPEECH_EFR: /* EFR */
-				memset(tch_data, 0, GSM_EFR_BYTES);
-				tch_data[0] = 0xc0;
-				rc = GSM_EFR_BYTES;
-				break;
-			case GSM48_CMODE_SPEECH_AMR: /* AMR */
-				rc = osmo_amr_rtp_enc(tch_data,
-					chan_state->codec[chan_state->ul_cmr],
-					chan_state->codec[chan_state->ul_ft],
-					AMR_BAD);
-				if (rc < 2) {
-					LOGL1SB(DL1P, LOGL_ERROR, l1ts, bi,
-					       "Failed to encode AMR_BAD frame (rc=%d), "
-					       "not sending BFI\n", rc);
-					return -EINVAL;
-				}
-				memset(tch_data + sizeof(struct amr_hdr), 0, rc - sizeof(struct amr_hdr));
-				break;
-			default:
-				LOGL1SB(DL1P, LOGL_ERROR, l1ts, bi,
-					"TCH mode %u invalid, please fix!\n", tch_mode);
-				return -EINVAL;
-			}
-		}
+		rc = 0;
 	}
 
-	if (rsl_cmode != RSL_CMOD_SPD_SPEECH)
+	if (rsl_cmode == RSL_CMOD_SPD_SIGN)
 		return 0;
 
 	/* TCH or BFI */
-compose_l1sap:
-	return _sched_compose_tch_ind(l1ts, fn_begin, bi->chan, tch_data, rc,
-				      meas_avg.toa256, ber10k, meas_avg.rssi,
-				      meas_avg.ci_cb, is_sub);
+	return _sched_compose_tch_ind(l1ts, fn_begin, bi->chan,
+				      &tch_data[0], rc,
+				      ber10k,
+				      meas_avg.rssi,
+				      meas_avg.toa256,
+				      meas_avg.ci_cb,
+				      is_sub);
 }
 
 /* common section for generation of TCH bursts (TCH/H and TCH/F).
  * FIXME: this function is over-complicated, refactor / get rid of it. */
-struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
+void tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br,
+		    struct msgb **msg_tch, struct msgb **msg_facch)
 {
-	struct msgb *msg1, *msg2, *msg_tch = NULL, *msg_facch = NULL;
+	struct msgb *msg1, *msg2;
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[br->chan];
 	uint8_t rsl_cmode = chan_state->rsl_cmode;
 	uint8_t tch_mode = chan_state->tch_mode;
@@ -351,50 +356,39 @@ struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br
 	if (msg1) {
 		l1sap = msgb_l1sap_prim(msg1);
 		if (l1sap->oph.primitive == PRIM_TCH) {
-			msg_tch = msg1;
+			*msg_tch = msg1;
 			if (msg2) {
 				l1sap = msgb_l1sap_prim(msg2);
 				if (l1sap->oph.primitive == PRIM_TCH) {
 					LOGL1SB(DL1P, LOGL_FATAL, l1ts, br, "TCH twice, please FIX!\n");
 					msgb_free(msg2);
 				} else
-					msg_facch = msg2;
+					*msg_facch = msg2;
 			}
 		} else {
-			msg_facch = msg1;
+			*msg_facch = msg1;
 			if (msg2) {
 				l1sap = msgb_l1sap_prim(msg2);
 				if (l1sap->oph.primitive != PRIM_TCH) {
 					LOGL1SB(DL1P, LOGL_FATAL, l1ts, br, "FACCH twice, please FIX!\n");
 					msgb_free(msg2);
 				} else
-					msg_tch = msg2;
+					*msg_tch = msg2;
 			}
 		}
 	}
 
 	/* check validity of message */
-	if (msg_facch && msgb_l2len(msg_facch) != GSM_MACBLOCK_LEN) {
+	if (*msg_facch != NULL && msgb_l2len(*msg_facch) != GSM_MACBLOCK_LEN) {
 		LOGL1SB(DL1P, LOGL_FATAL, l1ts, br, "Prim has odd len=%u != %u\n",
-			msgb_l2len(msg_facch), GSM_MACBLOCK_LEN);
+			msgb_l2len(*msg_facch), GSM_MACBLOCK_LEN);
 		/* free message */
-		msgb_free(msg_facch);
-		msg_facch = NULL;
-	}
-
-	/* prioritize FACCH over speech */
-	if (msg_facch) {
-		/* Unlike SACCH, FACCH has no dedicated slots on the multiframe layout.
-		 * It's multiplexed together with TCH (speech or data) frames basically
-		 * by replacing (stealing) them.  This is common for both TCH/F and
-		 * TCH/H, with the only difference that FACCH/H steals two TCH frames
-		 * (not just one) due to a longer interleaving period. */
-		msgb_free(msg_tch);
-		return msg_facch;
+		msgb_free(*msg_facch);
+		*msg_facch = NULL;
 	}
 
 	/* check validity of message, get AMR ft and cmr */
-	if (msg_tch) {
+	if (*msg_tch != NULL) {
 		int len;
 		uint8_t cmr_codec;
 		int ft, i;
@@ -403,8 +397,8 @@ struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br
 		int8_t sti, cmi;
 		bool amr_is_cmr;
 
-		if (rsl_cmode != RSL_CMOD_SPD_SPEECH) {
-			LOGL1SB(DL1P, LOGL_NOTICE, l1ts, br, "Dropping speech frame, "
+		if (OSMO_UNLIKELY(rsl_cmode == RSL_CMOD_SPD_SIGN)) {
+			LOGL1SB(DL1P, LOGL_NOTICE, l1ts, br, "Dropping a TCH frame, "
 				"because we are not in speech mode\n");
 			goto free_bad_msg;
 		}
@@ -412,7 +406,7 @@ struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br
 		switch (tch_mode) {
 		case GSM48_CMODE_SPEECH_V1: /* FR / HR */
 			if (br->chan != TRXC_TCHF) /* HR */
-				len = GSM_HR_BYTES + 1; /* RFC 5993 */
+				len = GSM_HR_BYTES;
 			else
 				len = GSM_FR_BYTES;
 			break;
@@ -422,7 +416,7 @@ struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br
 			len = GSM_EFR_BYTES;
 			break;
 		case GSM48_CMODE_SPEECH_AMR: /* AMR */
-			len = osmo_amr_rtp_dec(msg_tch->l2h, msgb_l2len(msg_tch),
+			len = osmo_amr_rtp_dec(msgb_l2((*msg_tch)), msgb_l2len(*msg_tch),
 					       &cmr_codec, &cmi, &ft_codec,
 					       &bfi, &sti);
 			if (len < 0) {
@@ -453,24 +447,47 @@ struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br
 				LOGL1SB(DL1P, LOGL_NOTICE, l1ts, br, "Transmitting 'bad AMR frame'\n");
 				goto free_bad_msg;
 			}
+			/* pull the AMR header, it's not being sent over Um */
+			(*msg_tch)->l2h += sizeof(struct amr_hdr);
+			len -= sizeof(struct amr_hdr);
+			break;
+		case GSM48_CMODE_DATA_14k5: /* TCH/F14.4 */
+			if (OSMO_UNLIKELY(br->chan != TRXC_TCHF))
+				goto inval_mode2;
+			len = 290;
+			break;
+		case GSM48_CMODE_DATA_12k0: /* TCH/F9.6 */
+			if (OSMO_UNLIKELY(br->chan != TRXC_TCHF))
+				goto inval_mode2;
+			len = 4 * 60;
+			break;
+		case GSM48_CMODE_DATA_6k0: /* TCH/[FH]4.8 */
+			if (br->chan == TRXC_TCHF)
+				len = 2 * 60;
+			else
+				len = 4 * 60;
+			break;
+		case GSM48_CMODE_DATA_3k6: /* TCH/[FH]2.4 */
+			if (br->chan == TRXC_TCHF)
+				len = 2 * 36;
+			else
+				len = 4 * 36;
 			break;
 		default:
 inval_mode2:
 			LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "TCH mode invalid, please fix!\n");
 			goto free_bad_msg;
 		}
-		if (msgb_l2len(msg_tch) != len) {
+		if (msgb_l2len(*msg_tch) != len) {
 			LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "Cannot send payload with "
 				"invalid length! (expecting %d, received %d)\n",
-				len, msgb_l2len(msg_tch));
+				len, msgb_l2len(*msg_tch));
 free_bad_msg:
 			/* free message */
-			msgb_free(msg_tch);
-			msg_tch = NULL;
+			msgb_free(*msg_tch);
+			*msg_tch = NULL;
 		}
 	}
-
-	return msg_tch;
 }
 
 /* obtain a to-be-transmitted TCH/F (Full Traffic Channel) burst */
@@ -478,70 +495,127 @@ int tx_tchf_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 {
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[br->chan];
 	uint8_t tch_mode = chan_state->tch_mode;
-	ubit_t *burst, **bursts_p = &chan_state->dl_bursts;
-	struct msgb *msg;
+	ubit_t *burst, *bursts_p = chan_state->dl_bursts;
+	uint8_t *mask = &chan_state->dl_mask;
+	struct msgb *msg_facch = NULL;
+	struct msgb *msg_tch = NULL;
+	struct msgb *msg = NULL;
 
 	/* send burst, if we already got a frame */
 	if (br->bid > 0) {
-		if (!*bursts_p)
-			return -ENODEV;
+		if ((*mask & 0x01) != 0x01)
+			return -ENOMSG;
 		goto send_burst;
 	}
 
+	*mask = *mask << 4;
+
 	/* BURST BYPASS */
 
-	/* allocate burst memory, if not already,
-	 * otherwise shift buffer by 4 bursts for interleaving */
-	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(l1ts, 928);
-		if (!*bursts_p)
-			return -ENOMEM;
-	} else {
-		memcpy(*bursts_p, *bursts_p + 464, 464);
-		memset(*bursts_p + 464, 0, 464);
-	}
+	 /* shift buffer by 4 bursts for interleaving */
+	memmove(BUFPOS(bursts_p, 0), BUFPOS(bursts_p, 4), 20 * BPLEN);
+	memset(BUFPOS(bursts_p, 20), 0, 4 * BPLEN);
 
-	/* dequeue a message to be transmitted */
-	msg = tch_dl_dequeue(l1ts, br);
-
-	/* no message at all, send a dummy L2 frame on FACCH */
-	if (msg == NULL) {
+	/* dequeue a TCH and/or a FACCH message to be transmitted */
+	tch_dl_dequeue(l1ts, br, &msg_tch, &msg_facch);
+	if (msg_tch == NULL && msg_facch == NULL) {
 		static const uint8_t dummy[GSM_MACBLOCK_LEN] = {
 			0x03, 0x03, 0x01, /* TODO: use randomized padding */
 			0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
 			0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
 		};
+		int rc;
 
 		LOGL1SB(DL1P, LOGL_DEBUG, l1ts, br, "No TCH or FACCH prim for transmit.\n");
-		gsm0503_tch_fr_encode(*bursts_p, dummy, sizeof(dummy), 1);
-		chan_state->dl_facch_bursts = 8;
+		/* If the channel mode is TCH/FS or TCH/EFS, transmit a dummy
+		 * speech block with inverted CRC3, designed to induce a BFI
+		 * condition in the MS receiver.  In all other channel modes,
+		 * transmit dummy FACCH like we always did before.
+		 *
+		 * FIXME: someone who knows AMR needs to look at this problem
+		 * and decide what is the correct BTS Tx behavior for frame
+		 * gaps in TCH/AFS.  See OS#6049.
+		 */
+		switch (tch_mode) {
+		case GSM48_CMODE_SPEECH_V1:
+		case GSM48_CMODE_SPEECH_EFR:
+			rc = gsm0503_tch_fr_encode(bursts_p, NULL, 0, 1);
+			if (rc == 0)
+				break;
+			/* fall-through */
+		default:
+			gsm0503_tch_fr_encode(bursts_p, dummy, sizeof(dummy), 1);
+			chan_state->dl_facch_bursts = 8;
+		}
 		goto send_burst;
 	}
 
-	/* populate the buffer with bursts */
-	if (msgb_l2len(msg) == GSM_MACBLOCK_LEN) {
-		gsm0503_tch_fr_encode(*bursts_p, msg->l2h, msgb_l2len(msg), 1);
+	/* Unlike SACCH, FACCH has no dedicated slots on the multiframe layout.
+	 * It's multiplexed together with TCH (speech or data) frames basically
+	 * by replacing (stealing) their bits, either completely or partly. */
+	msg = (msg_facch != NULL) ? msg_facch : msg_tch;
+	if (msg == msg_facch)
 		chan_state->dl_facch_bursts = 8;
-	} else if (tch_mode == GSM48_CMODE_SPEECH_AMR) {
+
+	/* populate the buffer with bursts */
+	switch (tch_mode) {
+	case GSM48_CMODE_SIGN:
+	case GSM48_CMODE_SPEECH_V1:
+	case GSM48_CMODE_SPEECH_EFR:
+		gsm0503_tch_fr_encode(BUFPOS(bursts_p, 0), msg->l2h, msgb_l2len(msg), 1);
+		break;
+	case GSM48_CMODE_SPEECH_AMR:
 		/* the first FN 4,13,21 defines that CMI is included in frame,
 		 * the first FN 0,8,17 defines that CMR is included in frame.
 		 */
-		gsm0503_tch_afs_encode(*bursts_p, msg->l2h + sizeof(struct amr_hdr),
-			msgb_l2len(msg) - sizeof(struct amr_hdr),
-			!sched_tchf_dl_amr_cmi_map[br->fn % 26],
-			chan_state->codec, chan_state->codecs,
-			chan_state->dl_ft,
-			chan_state->dl_cmr);
-	} else {
-		gsm0503_tch_fr_encode(*bursts_p, msg->l2h, msgb_l2len(msg), 1);
+		gsm0503_tch_afs_encode(BUFPOS(bursts_p, 0),
+				       msgb_l2(msg), msgb_l2len(msg),
+				       !sched_tchf_dl_amr_cmi_map[br->fn % 26],
+				       chan_state->codec,
+				       chan_state->codecs,
+				       chan_state->dl_ft,
+				       chan_state->dl_cmr);
+		break;
+	/* CSD (TCH/F9.6): 12.0 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_12k0:
+		if (msg_tch != NULL)
+			gsm0503_tch_fr96_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_tch));
+		if (msg_facch != NULL)
+			gsm0503_tch_fr_facch_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_facch));
+		break;
+	/* CSD (TCH/F4.8): 6.0 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_6k0:
+		if (msg_tch != NULL)
+			gsm0503_tch_fr48_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_tch));
+		if (msg_facch != NULL)
+			gsm0503_tch_fr_facch_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_facch));
+		break;
+	/* CSD (TCH/F2.4): 3.6 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_3k6:
+		/* FACCH/F does steal a TCH/F2.4 frame completely */
+		if (msg == msg_facch)
+			gsm0503_tch_fr_facch_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_facch));
+		else
+			gsm0503_tch_fr24_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_tch));
+		break;
+	/* CSD (TCH/F14.4): 14.5 kbit/s radio interface rate */
+	case GSM48_CMODE_DATA_14k5:
+		if (msg_tch != NULL)
+			gsm0503_tch_fr144_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_tch));
+		if (msg_facch != NULL)
+			gsm0503_tch_fr_facch_encode(BUFPOS(bursts_p, 0), msgb_l2(msg_facch));
+		break;
+	default:
+		OSMO_ASSERT(0);
 	}
 
-	/* free message */
-	msgb_free(msg);
+	/* free messages */
+	msgb_free(msg_tch);
+	msgb_free(msg_facch);
 
 send_burst:
 	/* compose burst */
-	burst = *bursts_p + br->bid * 116;
+	burst = BUFPOS(bursts_p, br->bid);
 	memcpy(br->burst + 3, burst, 58);
 	memcpy(br->burst + 61, TRX_GMSK_NB_TSC(br), 26);
 	memcpy(br->burst + 87, burst + 58, 58);
@@ -553,6 +627,8 @@ send_burst:
 		br->flags |= TRX_BR_F_FACCH;
 	}
 
+	*mask |= (1 << br->bid);
+
 	LOGL1SB(DL1P, LOGL_DEBUG, l1ts, br, "Transmitting burst=%u.\n", br->bid);
 
 	return 0;
