@@ -50,9 +50,9 @@ static void add_sbits(sbit_t * current, const sbit_t * previous)
 int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 {
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[bi->chan];
-	sbit_t *burst, **bursts_p = &chan_state->ul_bursts;
+	sbit_t *burst, *bursts_p = chan_state->ul_bursts;
 	uint32_t *first_fn = &chan_state->ul_first_fn;
-	uint8_t *mask = &chan_state->ul_mask;
+	uint32_t *mask = &chan_state->ul_mask;
 	uint8_t l2[GSM_MACBLOCK_LEN], l2_len;
 	struct l1sched_meas_set meas_avg;
 	int n_errors = 0;
@@ -69,24 +69,9 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi, "Received Data, bid=%u\n", bi->bid);
 
-	/* allocate burst memory, if not already */
-	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(l1ts, 464);
-		if (!*bursts_p)
-			return -ENOMEM;
-	}
-
-	/* UL-SACCH requires additional memory to keep a copy of each previous
-	 * burst set. */
-	if (L1SAP_IS_LINK_SACCH(trx_chan_desc[bi->chan].link_id) && !chan_state->ul_bursts_prev) {
-		chan_state->ul_bursts_prev = talloc_zero_size(l1ts, 464);
-		if (!chan_state->ul_bursts_prev)
-			return -ENOMEM;
-	}
-
 	/* clear burst & store frame number of first burst */
 	if (bi->bid == 0) {
-		memset(*bursts_p, 0, 464);
+		memset(bursts_p, 0, 464);
 		*mask = 0x0;
 		*first_fn = bi->fn;
 	}
@@ -99,12 +84,11 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	/* Copy burst to buffer of 4 bursts. If the burst indication contains
 	 * no data, ensure that the buffer does not stay uninitialized */
-	burst = *bursts_p + bi->bid * 116;
+	burst = bursts_p + bi->bid * 116;
 	if (bi->burst_len > 0) {
 		memcpy(burst, bi->burst + 3, 58);
 		memcpy(burst + 58, bi->burst + 87, 58);
-	} else
-		memset(burst, 0, 58 * 2);
+	}
 
 	/* wait until complete set of bursts */
 	if (bi->bid != 3)
@@ -127,10 +111,10 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	*mask = 0x0;
 
 	/* decode */
-	rc = gsm0503_xcch_decode(l2, *bursts_p, &n_errors, &n_bits_total);
+	rc = gsm0503_xcch_decode(l2, bursts_p, &n_errors, &n_bits_total);
 	if (rc) {
-		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi, "Received bad data (%u/%u)\n",
-			bi->fn % l1ts->mf_period, l1ts->mf_period);
+		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi,
+			BAD_DATA_MSG_FMT "\n", BAD_DATA_MSG_ARGS);
 		l2_len = 0;
 
 		/* When SACCH Repetition is active, we may try to decode the
@@ -138,8 +122,8 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		 * information from the previous SACCH block. See also:
 		 * 3GPP TS 44.006, section 11.2 */
 		if (rep_sacch) {
-			add_sbits(*bursts_p, chan_state->ul_bursts_prev);
-			rc = gsm0503_xcch_decode(l2, *bursts_p, &n_errors, &n_bits_total);
+			add_sbits(bursts_p, chan_state->ul_bursts_prev);
+			rc = gsm0503_xcch_decode(l2, bursts_p, &n_errors, &n_bits_total);
 			if (rc) {
 				LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi,
 				       "Combining current SACCH block with previous SACCH block also yields bad data (%u/%u)\n",
@@ -158,12 +142,14 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	/* Keep a copy to ease decoding in the next repetition pass */
 	if (rep_sacch)
-		memcpy(chan_state->ul_bursts_prev, *bursts_p, 464);
+		memcpy(chan_state->ul_bursts_prev, bursts_p, 464);
 
-	return _sched_compose_ph_data_ind(l1ts, *first_fn,
-					  bi->chan, l2, l2_len,
-					  meas_avg.rssi, meas_avg.toa256,
-					  meas_avg.ci_cb, ber10k,
+	return _sched_compose_ph_data_ind(l1ts, *first_fn, bi->chan,
+					  &l2[0], l2_len,
+					  ber10k,
+					  meas_avg.rssi,
+					  meas_avg.toa256,
+					  meas_avg.ci_cb,
 					  PRES_INFO_UNKNOWN);
 }
 
@@ -171,64 +157,54 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 int tx_data_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 {
 	struct msgb *msg = NULL; /* make GCC happy */
-	ubit_t *burst, **bursts_p = &l1ts->chan_state[br->chan].dl_bursts;
+	struct l1sched_chan_state *chan_state = &l1ts->chan_state[br->chan];
+	ubit_t *burst, *bursts_p = chan_state->dl_bursts;
+	uint8_t *mask = &chan_state->dl_mask;
 
 	/* send burst, if we already got a frame */
 	if (br->bid > 0) {
-		if (!*bursts_p)
-			return -ENODEV;
+		if ((*mask & 0x01) != 0x01)
+			return -ENOMSG;
 		goto send_burst;
 	}
 
+	*mask = *mask << 4;
+
 	/* get mac block from queue */
 	msg = _sched_dequeue_prim(l1ts, br);
-	if (msg)
-		goto got_msg;
-
-	LOGL1SB(DL1P, LOGL_INFO, l1ts, br, "No prim for transmit.\n");
-
-no_msg:
-	/* free burst memory */
-	if (*bursts_p) {
-		talloc_free(*bursts_p);
-		*bursts_p = NULL;
+	if (msg == NULL) {
+		LOGL1SB(DL1P, LOGL_INFO, l1ts, br, "No prim for transmit.\n");
+		return -ENODEV;
 	}
-	return -ENODEV;
 
-got_msg:
 	/* check validity of message */
 	if (msgb_l2len(msg) != GSM_MACBLOCK_LEN) {
 		LOGL1SB(DL1P, LOGL_FATAL, l1ts, br, "Prim has odd len=%u != %u\n",
 			msgb_l2len(msg), GSM_MACBLOCK_LEN);
 		/* free message */
 		msgb_free(msg);
-		goto no_msg;
+		return -EINVAL;
 	}
 
 	/* BURST BYPASS */
 
-	/* allocate burst memory, if not already */
-	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(l1ts, 464);
-		if (!*bursts_p)
-			return -ENOMEM;
-	}
-
 	/* encode bursts */
-	gsm0503_xcch_encode(*bursts_p, msg->l2h);
+	gsm0503_xcch_encode(bursts_p, msg->l2h);
 
 	/* free message */
 	msgb_free(msg);
 
 send_burst:
 	/* compose burst */
-	burst = *bursts_p + br->bid * 116;
+	burst = bursts_p + br->bid * 116;
 	memcpy(br->burst + 3, burst, 58);
 	memcpy(br->burst + 61, TRX_GMSK_NB_TSC(br), 26);
 	memcpy(br->burst + 87, burst + 58, 58);
 
 	br->burst_len = GSM_BURST_LEN;
 
+	*mask |= (1 << br->bid);
+
 	LOGL1SB(DL1P, LOGL_DEBUG, l1ts, br, "Transmitting burst=%u.\n", br->bid);
 
 	return 0;
