@@ -23,8 +23,24 @@
 
 #include <stdint.h>
 
+/* Burst Payload LENgth (short alias) */
+#define BPLEN GSM_NBITS_NB_GMSK_PAYLOAD
+
+/* Burst BUFfer capacity (in BPLEN units).  Why 24?  Because CSD frames
+ * are interleaved over 22 bursts, but on TCH/F we decode every 4th burst,
+ * so it must be a multiple of 4.  See also 'doc/trx_sched_tch.txt'. */
+#define BUFMAX 24
+
+/* Burst BUFfer position macros */
+#define BUFPOS(buf, n) &buf[(n) * BPLEN]
+#define BUFTAIL8(buf) BUFPOS(buf, (BUFMAX - 8))
+
 extern void *tall_bts_ctx;
 
+#define BAD_DATA_MSG_FMT "Received bad data (rc=%d, BER %d/%d) ending at fn=%u/%u"
+#define BAD_DATA_MSG_ARGS \
+	rc, n_errors, n_bits_total, bi->fn % l1ts->mf_period, l1ts->mf_period
+
 /* Compute the bit error rate in 1/10000 units */
 static inline uint16_t compute_ber10k(int n_bits_total, int n_errors)
 {
