@@ -8,7 +8,7 @@
  * Copyright (C) 2013  Andreas Eversberg <jolly@eversberg.eu>
  * Copyright (C) 2016-2017  Harald Welte <laforge@gnumonks.org>
  * Copyright (C) 2019  Vadim Yanitskiy <axilirator@gmail.com>
- * Copyright (C) 2021  sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+ * Copyright (C) 2021  sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  *
  * All Rights Reserved
  *
@@ -1232,7 +1232,8 @@ void trx_if_flush(struct trx_l1h *l1h)
 /*! close the TRX for given handle (data + control socket) */
 void trx_if_close(struct trx_l1h *l1h)
 {
-	LOGPPHI(l1h->phy_inst, DTRX, LOGL_NOTICE, "Closing TRXC/TRXD connections\n");
+	LOGPPHI(l1h->phy_inst, DTRX, LOGL_NOTICE, "Closing TRXC/TRXD connections to %s\n",
+		l1h->phy_inst->phy_link->u.osmotrx.remote_ip);
 
 	trx_if_flush(l1h);
 
@@ -1242,7 +1243,7 @@ void trx_if_close(struct trx_l1h *l1h)
 }
 
 /*! compute UDP port number used for TRX protocol */
-static uint16_t compute_port(struct phy_instance *pinst, int remote, int is_data)
+static uint16_t compute_port(struct phy_instance *pinst, bool remote, bool is_data)
 {
 	struct phy_link *plink = pinst->phy_link;
 	uint16_t inc = 1;
@@ -1263,21 +1264,21 @@ static int trx_if_open(struct trx_l1h *l1h)
 	struct phy_link *plink = pinst->phy_link;
 	int rc;
 
-	LOGPPHI(pinst, DTRX, LOGL_NOTICE, "Opening TRXC/TRXD connections\n");
+	LOGPPHI(pinst, DTRX, LOGL_NOTICE, "Opening TRXC/TRXD connections to %s\n", plink->u.osmotrx.remote_ip);
 
 	/* open sockets */
 	rc = trx_udp_open(l1h, &l1h->trx_ofd_ctrl,
 			  plink->u.osmotrx.local_ip,
-			  compute_port(pinst, 0, 0),
+			  compute_port(pinst, false, false),
 			  plink->u.osmotrx.remote_ip,
-			  compute_port(pinst, 1, 0), trx_ctrl_read_cb);
+			  compute_port(pinst, true, false), trx_ctrl_read_cb);
 	if (rc < 0)
 		return rc;
 	rc = trx_udp_open(l1h, &l1h->trx_ofd_data,
 			  plink->u.osmotrx.local_ip,
-			  compute_port(pinst, 0, 1),
+			  compute_port(pinst, false, true),
 			  plink->u.osmotrx.remote_ip,
-			  compute_port(pinst, 1, 1), trx_data_read_cb);
+			  compute_port(pinst, true, true), trx_data_read_cb);
 	if (rc < 0)
 		return rc;
 
