@@ -8,6 +8,7 @@
 #include <osmo-bts/gsm_data.h>
 #include <osmo-bts/logging.h>
 #include <osmo-bts/bts.h>
+#include <osmo-bts/bts_sm.h>
 #include <osmo-bts/measurement.h>
 #include <osmo-bts/rsl.h>
 
@@ -329,7 +330,7 @@ void test_lchan_meas_process_measurement(bool no_sacch, bool dropouts)
 
 	ulm.ber10k = 0;
 	ulm.ta_offs_256bits = 256;
-	ulm.c_i = 0;
+	ulm.ci_cb = 0;
 	ulm.is_sub = 0;
 	ulm.inv_rssi = 90;
 
@@ -373,73 +374,27 @@ void test_lchan_meas_process_measurement(bool no_sacch, bool dropouts)
 	}
 }
 
-static bool test_ts45008_83_is_sub_is_sacch(uint32_t fn)
-{
-	if (fn % 104 == 12)
-		return true;
-	if (fn % 104 == 25)
-		return true;
-	if (fn % 104 == 38)
-		return true;
-	if (fn % 104 == 51)
-		return true;
-	if (fn % 104 == 64)
-		return true;
-	if (fn % 104 == 77)
-		return true;
-	if (fn % 104 == 90)
-		return true;
-	if (fn % 104 == 103)
-		return true;
-
-	return false;
-}
-
-static bool test_ts45008_83_is_sub_is_sub(uint32_t fn, uint8_t ss)
+static bool test_ts45008_83_is_sub_is_sub(const struct gsm_lchan *lchan, uint32_t fn)
 {
 	fn = fn % 104;
 
-	if (fn >= 52 && fn <= 59)
-		return true;
-
-	if (ss == 0) {
-		if (fn == 0)
+	switch (lchan->type) {
+	case GSM_LCHAN_TCH_F:
+		/* block {52, 53, 54, 55, 56, 57, 58, 59} */
+		return fn == 52;
+	case GSM_LCHAN_TCH_H:
+		if (fn == 0)	/* H0 block { 0,  2,  4,  6} */
 			return true;
-		if (fn == 2)
+		if (fn == 52)	/* H0 block {52, 54, 56, 58} */
 			return true;
-		if (fn == 4)
+		if (fn == 14)	/* H1 block {14, 16, 18, 20} */
 			return true;
-		if (fn == 6)
+		if (fn == 66)	/* H1 block {66, 68, 70, 72} */
 			return true;
-		if (fn == 52)
-			return true;
-		if (fn == 54)
-			return true;
-		if (fn == 56)
-			return true;
-		if (fn == 58)
-			return true;
-	} else if (ss == 1) {
-		if (fn == 14)
-			return true;
-		if (fn == 16)
-			return true;
-		if (fn == 18)
-			return true;
-		if (fn == 20)
-			return true;
-		if (fn == 66)
-			return true;
-		if (fn == 68)
-			return true;
-		if (fn == 70)
-			return true;
-		if (fn == 72)
-			return true;
-	} else
-		OSMO_ASSERT(false);
-
-	return false;
+		return false;
+	default:
+		return false;
+	}
 }
 
 static void test_ts45008_83_is_sub_single(uint8_t ts, uint8_t ss, bool fr)
@@ -464,29 +419,17 @@ static void test_ts45008_83_is_sub_single(uint8_t ts, uint8_t ss, bool fr)
 		lchan->tch_mode = GSM48_CMODE_SPEECH_V1;
 	}
 
-	printf(" TS=%u ", ts);
-	printf("SS=%u", ss);
+	printf(" TS=%u SS=%u\n", ts, ss);
 
 	/* Walk trough the first 100 intervals and check for unexpected
 	 * results (false positive and false negative) */
 	for (i = 0; i < 104 * 100; i++) {
 		rc = ts45008_83_is_sub(lchan, i);
-		if (rc) {
-			if (!test_ts45008_83_is_sub_is_sacch(i)
-			    && !test_ts45008_83_is_sub_is_sub(i, ss)) {
-				printf("==> Unexpected SUB frame at fn=%u", i);
-				OSMO_ASSERT(false);
-			}
-		} else {
-			if (test_ts45008_83_is_sub_is_sacch(i)
-			    && test_ts45008_83_is_sub_is_sub(i, ss)) {
-				printf("==> Unexpected non-SUB frame at fn=%u",
-				       i);
-				OSMO_ASSERT(false);
-			}
+		if (rc != test_ts45008_83_is_sub_is_sub(lchan, i)) {
+			printf("  ==> ts45008_83_is_sub(fn=%u) yields %s, expected %s\n",
+			       i, rc ? "true" : "false", !rc ? "true" : "false");
 		}
 	}
-	printf("\n");
 }
 
 static void test_ts45008_83_is_sub(void)
@@ -515,7 +458,12 @@ int main(int argc, char **argv)
 	osmo_init_logging2(tall_bts_ctx, &bts_log_info);
 	osmo_stderr_target->categories[DMEAS].loglevel = LOGL_DEBUG;
 
-	bts = gsm_bts_alloc(tall_bts_ctx, 0);
+	g_bts_sm = gsm_bts_sm_alloc(tall_bts_ctx);
+	if (!g_bts_sm) {
+		fprintf(stderr, "Failed to create BTS Site Manager structure\n");
+		exit(1);
+	}
+	bts = gsm_bts_alloc(g_bts_sm, 0);
 	if (!bts) {
 		fprintf(stderr, "Failed to create BTS structure\n");
 		exit(1);
